import datetime
import typing
from collections.abc import Mapping, Sequence
from enum import StrEnum

import pydantic

from factorialhr._client import Endpoint, ListApiResponse, MetaApiResponse


class ExpenseRecord(pydantic.BaseModel):
    """Model for project_management_expense_record."""

    model_config = pydantic.ConfigDict(frozen=True)

    id: int = pydantic.Field(description='Expense record ID')
    project_worker_id: int = pydantic.Field(description='Project worker ID')
    expense_id: int = pydantic.Field(description='Expense ID')
    subproject_id: int | None = pydantic.Field(default=None, description='Subproject ID')
    original_amount_currency: str | None = pydantic.Field(default=None, description='Original amount currency')
    original_amount_cents: int | None = pydantic.Field(default=None, description='Original amount in cents')
    legal_entity_amount_currency: str | None = pydantic.Field(default=None, description='Legal entity amount currency')
    legal_entity_amount_cents: str | None = pydantic.Field(default=None, description='Legal entity amount in cents')
    effective_on: datetime.date | None = pydantic.Field(default=None, description='Effective date')
    exchange_rate: int | None = pydantic.Field(default=None, description='Exchange rate')
    status: str | None = pydantic.Field(default=None, description='Expense record status')


class ExpenseRecordEndpoint(Endpoint):
    endpoint = 'project_management/expense_records'

    async def all(self, **kwargs) -> ListApiResponse[ExpenseRecord]:
        """Get all expense records."""
        data = await self.api.get_all(self.endpoint, **kwargs)
        return ListApiResponse(model_type=ExpenseRecord, raw_data=data)

    async def get(self, **kwargs) -> MetaApiResponse[ExpenseRecord]:
        """Get expense records with pagination metadata."""
        query_params = kwargs.pop('params', {})
        query_params.setdefault('page', 1)
        response = await self.api.get(self.endpoint, params=query_params, **kwargs)
        return MetaApiResponse(model_type=ExpenseRecord, raw_meta=response['meta'], raw_data=response['data'])

    async def get_by_id(self, expense_id: int | str, **kwargs) -> ExpenseRecord:
        """Get a specific expense record by ID."""
        data = await self.api.get(self.endpoint, expense_id, **kwargs)
        return pydantic.TypeAdapter(ExpenseRecord).validate_python(data['data'])

    async def create(self, data: Mapping[str, typing.Any], **kwargs) -> ExpenseRecord:
        """Create a new expense record."""
        response = await self.api.post(self.endpoint, json=data, **kwargs)
        return pydantic.TypeAdapter(ExpenseRecord).validate_python(response['data'])

    async def update(self, expense_id: int | str, data: Mapping[str, typing.Any], **kwargs) -> ExpenseRecord:
        """Update an expense record."""
        response = await self.api.put(self.endpoint, expense_id, json=data, **kwargs)
        return pydantic.TypeAdapter(ExpenseRecord).validate_python(response['data'])

    async def delete(self, expense_id: int | str, **kwargs) -> None:
        """Delete an expense record."""
        await self.api.delete(self.endpoint, expense_id, **kwargs)


class ExportableExpense(pydantic.BaseModel):
    """Model for project_management_exportable_expense."""

    model_config = pydantic.ConfigDict(frozen=True)

    date: datetime.date | None = pydantic.Field(default=None, description='Expense date')
    project_name: str | None = pydantic.Field(default=None, description='Project name')
    subproject_name: str | None = pydantic.Field(default=None, description='Subproject name')
    employee_name: str = pydantic.Field(description='Employee name')
    preferred_name: str | None = pydantic.Field(default=None, description='Employee preferred name')
    amount: str | None = pydantic.Field(default=None, description='Expense amount')
    currency: str | None = pydantic.Field(default=None, description='Currency')
    expense_category: str | None = pydantic.Field(default=None, description='Expense category')
    expense_subcategory: str | None = pydantic.Field(default=None, description='Expense subcategory')
    expense_status: str | None = pydantic.Field(default=None, description='Expense status')
    expense_link: str | None = pydantic.Field(default=None, description='Link to expense details')


class ExportableExpenseEndpoint(Endpoint):
    endpoint = 'project_management/exportable_expenses'

    async def all(self, **kwargs) -> ListApiResponse[ExportableExpense]:
        """Get all exportable expenses."""
        data = await self.api.get_all(self.endpoint, **kwargs)
        return ListApiResponse(model_type=ExportableExpense, raw_data=data)

    async def get(self, **kwargs) -> MetaApiResponse[ExportableExpense]:
        """Get exportable expenses with pagination metadata."""
        query_params = kwargs.pop('params', {})
        query_params.setdefault('page', 1)
        response = await self.api.get(self.endpoint, params=query_params, **kwargs)
        return MetaApiResponse(model_type=ExportableExpense, raw_meta=response['meta'], raw_data=response['data'])

    async def get_by_id(self, expense_id: int | str, **kwargs) -> ExportableExpense:
        """Get a specific exportable expense by ID."""
        data = await self.api.get(self.endpoint, expense_id, **kwargs)
        return pydantic.TypeAdapter(ExportableExpense).validate_python(data['data'])

    async def create(self, data: Mapping[str, typing.Any], **kwargs) -> ExportableExpense:
        """Create a new exportable expense."""
        response = await self.api.post(self.endpoint, json=data, **kwargs)
        return pydantic.TypeAdapter(ExportableExpense).validate_python(response['data'])

    async def update(self, expense_id: int | str, data: Mapping[str, typing.Any], **kwargs) -> ExportableExpense:
        """Update an exportable expense."""
        response = await self.api.put(self.endpoint, expense_id, json=data, **kwargs)
        return pydantic.TypeAdapter(ExportableExpense).validate_python(response['data'])

    async def delete(self, expense_id: int | str, **kwargs) -> None:
        """Delete an exportable expense."""
        await self.api.delete(self.endpoint, expense_id, **kwargs)


class ExportableProject(pydantic.BaseModel):
    """Model for project_management_exportable_project."""

    model_config = pydantic.ConfigDict(frozen=True)

    id: str = pydantic.Field(description='Project ID')
    date: datetime.date | None = pydantic.Field(default=None, description='Project date')
    project_name: str = pydantic.Field(description='Project name')
    project_code: str | None = pydantic.Field(default=None, description='Project code')
    project_start_date: datetime.date | None = pydantic.Field(default=None, description='Project start date')
    project_due_date: datetime.date | None = pydantic.Field(default=None, description='Project due date')
    project_status: str = pydantic.Field(description='Project status')
    subproject_name: str | None = pydantic.Field(default=None, description='Subproject name')
    employee_name: str | None = pydantic.Field(default=None, description='Employee name')
    employee_id: int | None = pydantic.Field(default=None, description='Employee ID')
    inputed_time: float = pydantic.Field(description='Inputted time (API returns string but can be float)')


class ExportableProjectEndpoint(Endpoint):
    endpoint = 'project_management/exportable_projects'

    async def all(self, **kwargs) -> ListApiResponse[ExportableProject]:
        """Get all exportable projects."""
        data = await self.api.get_all(self.endpoint, **kwargs)
        return ListApiResponse(model_type=ExportableProject, raw_data=data)

    async def get(self, **kwargs) -> MetaApiResponse[ExportableProject]:
        """Get exportable projects with pagination metadata."""
        query_params = kwargs.pop('params', {})
        query_params.setdefault('page', 1)
        response = await self.api.get(self.endpoint, params=query_params, **kwargs)
        return MetaApiResponse(model_type=ExportableProject, raw_meta=response['meta'], raw_data=response['data'])

    async def get_by_id(self, project_id: int | str, **kwargs) -> ExportableProject:
        """Get a specific exportable project by ID."""
        data = await self.api.get(self.endpoint, project_id, **kwargs)
        return pydantic.TypeAdapter(ExportableProject).validate_python(data['data'])

    async def create(self, data: Mapping[str, typing.Any], **kwargs) -> ExportableProject:
        """Create a new exportable project."""
        response = await self.api.post(self.endpoint, json=data, **kwargs)
        return pydantic.TypeAdapter(ExportableProject).validate_python(response['data'])

    async def update(self, project_id: int | str, data: Mapping[str, typing.Any], **kwargs) -> ExportableProject:
        """Update an exportable project."""
        response = await self.api.put(self.endpoint, project_id, json=data, **kwargs)
        return pydantic.TypeAdapter(ExportableProject).validate_python(response['data'])

    async def delete(self, project_id: int | str, **kwargs) -> None:
        """Delete an exportable project."""
        await self.api.delete(self.endpoint, project_id, **kwargs)


class FlexibleTimeRecord(pydantic.BaseModel):
    """Model for project_management_flexible_time_record."""

    model_config = pydantic.ConfigDict(frozen=True)

    id: int = pydantic.Field(description='Flexible time record ID')
    date: datetime.date = pydantic.Field(description='Record date')
    imputed_minutes: int = pydantic.Field(description='Imputed minutes')
    project_worker_id: int = pydantic.Field(description='Project worker ID')
    subproject_id: int | None = pydantic.Field(default=None, description='Subproject ID')


class FlexibleTimeRecordEndpoint(Endpoint):
    endpoint = 'project_management/flexible_time_records'

    async def all(self, **kwargs) -> ListApiResponse[FlexibleTimeRecord]:
        """Get all flexible time records."""
        data = await self.api.get_all(self.endpoint, **kwargs)
        return ListApiResponse(model_type=FlexibleTimeRecord, raw_data=data)

    async def get(self, **kwargs) -> MetaApiResponse[FlexibleTimeRecord]:
        """Get flexible time records with pagination metadata."""
        query_params = kwargs.pop('params', {})
        query_params.setdefault('page', 1)
        response = await self.api.get(self.endpoint, params=query_params, **kwargs)
        return MetaApiResponse(model_type=FlexibleTimeRecord, raw_meta=response['meta'], raw_data=response['data'])

    async def get_by_id(self, record_id: int | str, **kwargs) -> FlexibleTimeRecord:
        """Get a specific flexible time record by ID."""
        data = await self.api.get(self.endpoint, record_id, **kwargs)
        return pydantic.TypeAdapter(FlexibleTimeRecord).validate_python(data['data'])

    async def create(self, data: Mapping[str, typing.Any], **kwargs) -> FlexibleTimeRecord:
        """Create a new flexible time record."""
        response = await self.api.post(self.endpoint, json=data, **kwargs)
        return pydantic.TypeAdapter(FlexibleTimeRecord).validate_python(response['data'])

    async def update(self, record_id: int | str, data: Mapping[str, typing.Any], **kwargs) -> FlexibleTimeRecord:
        """Update a flexible time record."""
        response = await self.api.put(self.endpoint, record_id, json=data, **kwargs)
        return pydantic.TypeAdapter(FlexibleTimeRecord).validate_python(response['data'])

    async def delete(self, record_id: int | str, **kwargs) -> FlexibleTimeRecord:
        """Delete a flexible time record."""
        response = await self.api.delete(self.endpoint, record_id, **kwargs)
        return pydantic.TypeAdapter(FlexibleTimeRecord).validate_python(response)


class FlexibleTimeRecordComment(pydantic.BaseModel):
    """Model for project_management_flexible_time_record_comment."""

    model_config = pydantic.ConfigDict(frozen=True)

    id: int = pydantic.Field(description='Comment ID')
    content: str = pydantic.Field(description='Comment content')
    flexible_time_record_id: int = pydantic.Field(description='Flexible time record ID')


class FlexibleTimeRecordCommentEndpoint(Endpoint):
    endpoint = 'project_management/flexible_time_record_comments'

    async def all(self, **kwargs) -> ListApiResponse[FlexibleTimeRecordComment]:
        """Get all flexible time record comments."""
        data = await self.api.get_all(self.endpoint, **kwargs)
        return ListApiResponse(model_type=FlexibleTimeRecordComment, raw_data=data)

    async def get(self, **kwargs) -> MetaApiResponse[FlexibleTimeRecordComment]:
        """Get flexible time record comments with pagination metadata."""
        query_params = kwargs.pop('params', {})
        query_params.setdefault('page', 1)
        response = await self.api.get(self.endpoint, params=query_params, **kwargs)
        return MetaApiResponse(
            model_type=FlexibleTimeRecordComment,
            raw_meta=response['meta'],
            raw_data=response['data'],
        )

    async def get_by_id(self, comment_id: int | str, **kwargs) -> FlexibleTimeRecordComment:
        """Get a specific flexible time record comment by ID."""
        data = await self.api.get(self.endpoint, comment_id, **kwargs)
        return pydantic.TypeAdapter(FlexibleTimeRecordComment).validate_python(data['data'])

    async def create(self, data: Mapping[str, typing.Any], **kwargs) -> FlexibleTimeRecordComment:
        """Create a new flexible time record comment."""
        response = await self.api.post(self.endpoint, json=data, **kwargs)
        return pydantic.TypeAdapter(FlexibleTimeRecordComment).validate_python(response['data'])

    async def update(
        self,
        comment_id: int | str,
        data: Mapping[str, typing.Any],
        **kwargs,
    ) -> FlexibleTimeRecordComment:
        """Update a flexible time record comment."""
        response = await self.api.put(self.endpoint, comment_id, json=data, **kwargs)
        return pydantic.TypeAdapter(FlexibleTimeRecordComment).validate_python(response['data'])

    async def delete(self, comment_id: int | str, **kwargs) -> FlexibleTimeRecordComment:
        """Delete a flexible time record comment."""
        response = await self.api.delete(self.endpoint, comment_id, **kwargs)
        return pydantic.TypeAdapter(FlexibleTimeRecordComment).validate_python(response)

    async def delete_by_flexible_time_record(
        self,
        data: Mapping[str, typing.Any],
        **kwargs,
    ) -> FlexibleTimeRecordComment:
        """Delete a flexible time record comment by flexible time record ID."""
        response = await self.api.post(self.endpoint, 'delete_by_flexible_time_record', json=data, **kwargs)
        return pydantic.TypeAdapter(FlexibleTimeRecordComment).validate_python(response)

    async def update_by_flexible_time_record(
        self,
        data: Mapping[str, typing.Any],
        **kwargs,
    ) -> FlexibleTimeRecordComment:
        """Update a flexible time record comment by flexible time record ID."""
        response = await self.api.post(self.endpoint, 'update_by_flexible_time_record', json=data, **kwargs)
        return pydantic.TypeAdapter(FlexibleTimeRecordComment).validate_python(response)


class ProjectStatus(StrEnum):
    """Enum for project status."""

    ACTIVE = 'active'
    CLOSED = 'closed'
    DRAFT = 'draft'
    PROCESSING = 'processing'


class ProjectEmployeeAssignment(StrEnum):
    """Enum for project employee assignment types."""

    MANUAL = 'manual'
    COMPANY = 'company'


class Project(pydantic.BaseModel):
    """Model for project_management_project."""

    model_config = pydantic.ConfigDict(frozen=True)

    id: int = pydantic.Field(description='Project ID')
    name: str = pydantic.Field(description='Project name')
    code: str | None = pydantic.Field(default=None, description='Project code')
    start_date: datetime.date | None = pydantic.Field(default=None, description='Project start date')
    due_date: datetime.date | None = pydantic.Field(default=None, description='Project due date')
    status: ProjectStatus = pydantic.Field(description='Project status')
    employees_assignment: ProjectEmployeeAssignment = pydantic.Field(description='Employee assignment type')
    inputed_minutes: int | None = pydantic.Field(default=None, description='Total inputted minutes')
    is_billable: bool = pydantic.Field(description='Whether the project is billable')
    fixed_cost_cents: int | None = pydantic.Field(default=None, description='Fixed cost in cents')
    labor_cost_cents: int | None = pydantic.Field(default=None, description='Labor cost in cents')
    legal_entity_id: int = pydantic.Field(description='Legal entity ID')
    spending_cost_cents: int | None = pydantic.Field(default=None, description='Spending cost in cents')
    client_id: int | None = pydantic.Field(default=None, description='Client ID')
    billable_rate_type: str | None = pydantic.Field(default=None, description='Billable rate type')
    total_cost_cents: int | None = pydantic.Field(default=None, description='Total cost in cents')


class ProjectEndpoint(Endpoint):
    endpoint = 'project_management/projects'

    async def all(self, **kwargs) -> ListApiResponse[Project]:
        """Get all projects."""
        data = await self.api.get_all(self.endpoint, **kwargs)
        return ListApiResponse(model_type=Project, raw_data=data)

    async def get(self, **kwargs) -> MetaApiResponse[Project]:
        """Get projects with pagination metadata."""
        query_params = kwargs.pop('params', {})
        query_params.setdefault('page', 1)
        response = await self.api.get(self.endpoint, params=query_params, **kwargs)
        return MetaApiResponse(model_type=Project, raw_meta=response['meta'], raw_data=response['data'])

    async def get_by_id(self, project_id: int | str, **kwargs) -> Project:
        """Get a specific project by ID."""
        data = await self.api.get(self.endpoint, project_id, **kwargs)
        return pydantic.TypeAdapter(Project).validate_python(data['data'])

    async def create(self, data: Mapping[str, typing.Any], **kwargs) -> Project:
        """Create a new project."""
        response = await self.api.post(self.endpoint, json=data, **kwargs)
        return pydantic.TypeAdapter(Project).validate_python(response['data'])

    async def update(self, project_id: int | str, data: Mapping[str, typing.Any], **kwargs) -> Project:
        """Update a project."""
        response = await self.api.put(self.endpoint, project_id, json=data, **kwargs)
        return pydantic.TypeAdapter(Project).validate_python(response['data'])

    async def delete(self, project_id: int | str, **kwargs) -> Project:
        """Delete a project."""
        response = await self.api.delete(self.endpoint, project_id, **kwargs)
        return pydantic.TypeAdapter(Project).validate_python(response)

    async def activate(self, data: Mapping[str, typing.Any], **kwargs) -> Project:
        """Activate a project."""
        response = await self.api.post(self.endpoint, 'activate', json=data, **kwargs)
        return pydantic.TypeAdapter(Project).validate_python(response)

    async def change_assignment(self, data: Mapping[str, typing.Any], **kwargs) -> Project:
        """Change assignment of a project."""
        response = await self.api.post(self.endpoint, 'change_assignment', json=data, **kwargs)
        return pydantic.TypeAdapter(Project).validate_python(response)

    async def change_status(self, data: Mapping[str, typing.Any], **kwargs) -> Project:
        """Change status of a project."""
        response = await self.api.post(self.endpoint, 'change_status', json=data, **kwargs)
        return pydantic.TypeAdapter(Project).validate_python(response)

    async def close(self, data: Mapping[str, typing.Any], **kwargs) -> Project:
        """Close a project."""
        response = await self.api.post(self.endpoint, 'close', json=data, **kwargs)
        return pydantic.TypeAdapter(Project).validate_python(response)

    async def soft_delete(self, data: Mapping[str, typing.Any], **kwargs) -> Project:
        """Soft delete a project."""
        response = await self.api.post(self.endpoint, 'soft_delete', json=data, **kwargs)
        return pydantic.TypeAdapter(Project).validate_python(response)


class ProjectTask(pydantic.BaseModel):
    """Model for project_management_project_task."""

    model_config = pydantic.ConfigDict(frozen=True)

    id: int = pydantic.Field(description='Project task ID')
    project_id: int = pydantic.Field(description='Project ID')
    subproject_id: int = pydantic.Field(description='Subproject ID')
    task_id: int = pydantic.Field(description='Task ID')
    follow_up: bool = pydantic.Field(description='Whether this is a follow-up task')


class ProjectTaskEndpoint(Endpoint):
    endpoint = 'project_management/project_tasks'

    async def all(self, **kwargs) -> ListApiResponse[ProjectTask]:
        """Get all project tasks."""
        data = await self.api.get_all(self.endpoint, **kwargs)
        return ListApiResponse(model_type=ProjectTask, raw_data=data)

    async def get(self, **kwargs) -> MetaApiResponse[ProjectTask]:
        """Get project tasks with pagination metadata."""
        query_params = kwargs.pop('params', {})
        query_params.setdefault('page', 1)
        response = await self.api.get(self.endpoint, params=query_params, **kwargs)
        return MetaApiResponse(model_type=ProjectTask, raw_meta=response['meta'], raw_data=response['data'])

    async def get_by_id(self, task_id: int | str, **kwargs) -> ProjectTask:
        """Get a specific project task by ID."""
        data = await self.api.get(self.endpoint, task_id, **kwargs)
        return pydantic.TypeAdapter(ProjectTask).validate_python(data['data'])

    async def create(self, data: Mapping[str, typing.Any], **kwargs) -> ProjectTask:
        """Create a new project task."""
        response = await self.api.post(self.endpoint, json=data, **kwargs)
        return pydantic.TypeAdapter(ProjectTask).validate_python(response['data'])

    async def update(self, task_id: int | str, data: Mapping[str, typing.Any], **kwargs) -> ProjectTask:
        """Update a project task."""
        response = await self.api.put(self.endpoint, task_id, json=data, **kwargs)
        return pydantic.TypeAdapter(ProjectTask).validate_python(response['data'])

    async def delete(self, task_id: int | str, **kwargs) -> ProjectTask:
        """Delete a project task."""
        response = await self.api.delete(self.endpoint, task_id, **kwargs)
        return pydantic.TypeAdapter(ProjectTask).validate_python(response)

    async def bulk_destroy(self, data: Mapping[str, typing.Any], **kwargs) -> Sequence[ProjectTask]:
        """Bulk destroy project tasks."""
        response = await self.api.post(self.endpoint, 'bulk_destroy', json=data, **kwargs)
        return pydantic.TypeAdapter(list[ProjectTask]).validate_python(response)

    async def bulk_duplicate(self, data: Mapping[str, typing.Any], **kwargs) -> Sequence[ProjectTask]:
        """Bulk duplicate project tasks."""
        response = await self.api.post(self.endpoint, 'bulk_duplicate', json=data, **kwargs)
        return pydantic.TypeAdapter(list[ProjectTask]).validate_python(response)


class ProjectWorker(pydantic.BaseModel):
    """Model for project_management_project_worker."""

    model_config = pydantic.ConfigDict(frozen=True)

    id: int = pydantic.Field(description='Project worker ID')
    project_id: int = pydantic.Field(description='Project ID')
    employee_id: int = pydantic.Field(description='Employee ID')
    assigned: bool = pydantic.Field(description='Whether the worker is assigned to the project')
    inputed_minutes: int | None = pydantic.Field(default=None, description='Total inputted minutes')
    labor_cost_cents: int | None = pydantic.Field(default=None, description='Labor cost in cents')
    spending_cost_cents: int | None = pydantic.Field(default=None, description='Spending cost in cents')


class ProjectWorkerEndpoint(Endpoint):
    endpoint = 'project_management/project_workers'

    async def all(self, **kwargs) -> ListApiResponse[ProjectWorker]:
        """Get all project workers."""
        data = await self.api.get_all(self.endpoint, **kwargs)
        return ListApiResponse(model_type=ProjectWorker, raw_data=data)

    async def get(self, **kwargs) -> MetaApiResponse[ProjectWorker]:
        """Get project workers with pagination metadata."""
        query_params = kwargs.pop('params', {})
        query_params.setdefault('page', 1)
        response = await self.api.get(self.endpoint, params=query_params, **kwargs)
        return MetaApiResponse(model_type=ProjectWorker, raw_meta=response['meta'], raw_data=response['data'])

    async def get_by_id(self, worker_id: int | str, **kwargs) -> ProjectWorker:
        """Get a specific project worker by ID."""
        data = await self.api.get(self.endpoint, worker_id, **kwargs)
        return pydantic.TypeAdapter(ProjectWorker).validate_python(data['data'])

    async def create(self, data: Mapping[str, typing.Any], **kwargs) -> ProjectWorker:
        """Create a new project worker."""
        response = await self.api.post(self.endpoint, json=data, **kwargs)
        return pydantic.TypeAdapter(ProjectWorker).validate_python(response['data'])

    async def update(self, worker_id: int | str, data: Mapping[str, typing.Any], **kwargs) -> ProjectWorker:
        """Update a project worker."""
        response = await self.api.put(self.endpoint, worker_id, json=data, **kwargs)
        return pydantic.TypeAdapter(ProjectWorker).validate_python(response['data'])

    async def delete(self, worker_id: int | str, **kwargs) -> ProjectWorker:
        """Delete a project worker."""
        response = await self.api.delete(self.endpoint, worker_id, **kwargs)
        return pydantic.TypeAdapter(ProjectWorker).validate_python(response)

    async def bulk_assign(self, data: Mapping[str, typing.Any], **kwargs) -> Sequence[ProjectWorker]:
        """Bulk assign project workers."""
        response = await self.api.post(self.endpoint, 'bulk_assign', json=data, **kwargs)
        return pydantic.TypeAdapter(list[ProjectWorker]).validate_python(response)

    async def bulk_create(self, data: Mapping[str, typing.Any], **kwargs) -> Sequence[ProjectWorker]:
        """Bulk create project workers."""
        response = await self.api.post(self.endpoint, 'bulk_create', json=data, **kwargs)
        return pydantic.TypeAdapter(list[ProjectWorker]).validate_python(response)

    async def unassign(self, data: Mapping[str, typing.Any], **kwargs) -> ProjectWorker:
        """Unassign a project worker."""
        response = await self.api.post(self.endpoint, 'unassign', json=data, **kwargs)
        return pydantic.TypeAdapter(ProjectWorker).validate_python(response)


class Subproject(pydantic.BaseModel):
    """Model for project_management_subproject."""

    model_config = pydantic.ConfigDict(frozen=True)

    id: int | None = pydantic.Field(default=None, description='Subproject ID')
    name: str = pydantic.Field(description='Subproject name')
    project_id: int = pydantic.Field(description='Project ID')
    inputed_minutes: int | None = pydantic.Field(default=None, description='Total inputted minutes')
    labor_cost_cents: int | None = pydantic.Field(default=None, description='Labor cost in cents')


class SubprojectEndpoint(Endpoint):
    endpoint = 'project_management/subprojects'

    async def all(self, **kwargs) -> ListApiResponse[Subproject]:
        """Get all subprojects."""
        data = await self.api.get_all(self.endpoint, **kwargs)
        return ListApiResponse(model_type=Subproject, raw_data=data)

    async def get(self, **kwargs) -> MetaApiResponse[Subproject]:
        """Get subprojects with pagination metadata."""
        query_params = kwargs.pop('params', {})
        query_params.setdefault('page', 1)
        response = await self.api.get(self.endpoint, params=query_params, **kwargs)
        return MetaApiResponse(model_type=Subproject, raw_meta=response['meta'], raw_data=response['data'])

    async def get_by_id(self, subproject_id: int | str, **kwargs) -> Subproject:
        """Get a specific subproject by ID."""
        data = await self.api.get(self.endpoint, subproject_id, **kwargs)
        return pydantic.TypeAdapter(Subproject).validate_python(data['data'])

    async def create(self, data: Mapping[str, typing.Any], **kwargs) -> Subproject:
        """Create a new subproject."""
        response = await self.api.post(self.endpoint, json=data, **kwargs)
        return pydantic.TypeAdapter(Subproject).validate_python(response['data'])

    async def update(self, subproject_id: int | str, data: Mapping[str, typing.Any], **kwargs) -> Subproject:
        """Update a subproject."""
        response = await self.api.put(self.endpoint, subproject_id, json=data, **kwargs)
        return pydantic.TypeAdapter(Subproject).validate_python(response['data'])

    async def delete(self, subproject_id: int | str, **kwargs) -> Subproject:
        """Delete a subproject."""
        response = await self.api.delete(self.endpoint, subproject_id, **kwargs)
        return pydantic.TypeAdapter(Subproject).validate_python(response)

    async def rename(self, data: Mapping[str, typing.Any], **kwargs) -> Subproject:
        """Rename a subproject."""
        response = await self.api.post(self.endpoint, 'rename', json=data, **kwargs)
        return pydantic.TypeAdapter(Subproject).validate_python(response)


class TimeRecord(pydantic.BaseModel):
    """Model for project_management_time_record."""

    model_config = pydantic.ConfigDict(frozen=True)

    id: int = pydantic.Field(description='Time record ID')
    project_worker_id: int = pydantic.Field(description='Project worker ID')
    attendance_shift_id: int = pydantic.Field(description='Attendance shift ID')
    subproject_id: int | None = pydantic.Field(default=None, description='Subproject ID')
    date: datetime.date | None = pydantic.Field(default=None, description='Record date')
    imputed_minutes: int | None = pydantic.Field(default=None, description='Imputed minutes')
    clock_in: datetime.datetime | None = pydantic.Field(
        default=None,
        description='Clock in time (date will always be 2000-01-01, only use for .time())',
    )
    clock_out: datetime.datetime | None = pydantic.Field(
        default=None,
        description='Clock out time (date will always be 2000-01-01, only use for .time())',
    )


class TimeRecordEndpoint(Endpoint):
    endpoint = 'project_management/time_records'

    async def all(self, **kwargs) -> ListApiResponse[TimeRecord]:
        """Get all time records."""
        data = await self.api.get_all(self.endpoint, **kwargs)
        return ListApiResponse(model_type=TimeRecord, raw_data=data)

    async def get(self, **kwargs) -> MetaApiResponse[TimeRecord]:
        """Get time records with pagination metadata."""
        query_params = kwargs.pop('params', {})
        query_params.setdefault('page', 1)
        response = await self.api.get(self.endpoint, params=query_params, **kwargs)
        return MetaApiResponse(model_type=TimeRecord, raw_meta=response['meta'], raw_data=response['data'])

    async def get_by_id(self, record_id: int | str, **kwargs) -> TimeRecord:
        """Get a specific time record by ID."""
        data = await self.api.get(self.endpoint, record_id, **kwargs)
        return pydantic.TypeAdapter(TimeRecord).validate_python(data['data'])

    async def create(self, data: Mapping[str, typing.Any], **kwargs) -> TimeRecord:
        """Create a new time record."""
        response = await self.api.post(self.endpoint, json=data, **kwargs)
        return pydantic.TypeAdapter(TimeRecord).validate_python(response['data'])

    async def update(self, record_id: int | str, data: Mapping[str, typing.Any], **kwargs) -> TimeRecord:
        """Update a time record."""
        response = await self.api.put(self.endpoint, record_id, json=data, **kwargs)
        return pydantic.TypeAdapter(TimeRecord).validate_python(response['data'])

    async def delete(self, record_id: int | str, **kwargs) -> TimeRecord:
        """Delete a time record."""
        response = await self.api.delete(self.endpoint, record_id, **kwargs)
        return pydantic.TypeAdapter(TimeRecord).validate_python(response)

    async def bulk_delete(self, data: Mapping[str, typing.Any], **kwargs) -> Sequence[TimeRecord]:
        """Bulk delete time records."""
        response = await self.api.post(self.endpoint, 'bulk_delete', json=data, **kwargs)
        return pydantic.TypeAdapter(list[TimeRecord]).validate_python(response)

    async def bulk_process(self, data: Mapping[str, typing.Any], **kwargs) -> Sequence[TimeRecord]:
        """Bulk process time records."""
        response = await self.api.post(self.endpoint, 'bulk_process', json=data, **kwargs)
        return pydantic.TypeAdapter(list[TimeRecord]).validate_python(response)

    async def update_project_worker(self, data: Mapping[str, typing.Any], **kwargs) -> TimeRecord:
        """Update project worker for a time record."""
        response = await self.api.post(self.endpoint, 'update_project_worker', json=data, **kwargs)
        return pydantic.TypeAdapter(TimeRecord).validate_python(response)



from enum import StrEnum

import pydantic

from factorialhr._client import Endpoint, ListApiResponse, MetaApiResponse


class ConceptCategory(StrEnum):
    """Enum for concept categories."""

    EARNINGS_FIXED_SALARY = 'earnings_fixed_salary'
    EARNINGS_VARIABLE = 'earnings_variable'
    EARNINGS_BENEFITS_IN_KIND = 'earnings_benefits_in_kind'
    EARNINGS_OTHERS = 'earnings_others'
    DEDUCTIONS = 'deductions'
    COMPANY_CONTRIBUTION = 'company_contribution'
    SUMMARIZED_VALUES = 'summarized_values'


class UnitType(StrEnum):
    """Enum for unit types."""

    DISTANCE = 'distance'
    MONEY = 'money'
    TIME = 'time'
    UNIT = 'unit'


class Concept(pydantic.BaseModel):
    """Model for compensations_concept."""

    model_config = pydantic.ConfigDict(frozen=True)

    id: int = pydantic.Field(description='The identifier of the concept')
    company_id: int = pydantic.Field(description='The company identifier of the concept')
    default: bool = pydantic.Field(description='Whether the concept is a default or a custom concept')
    description: str = pydantic.Field(description='The description of the concept')
    label: str = pydantic.Field(description='The label of the concept')
    name: str = pydantic.Field(description='The name of the concept')
    category: ConceptCategory | None = pydantic.Field(default=None, description='The category of the concept')
    translated_name: str | None = pydantic.Field(
        default=None,
        description='The translated name of the concept if it is a default concept',
    )
    unit_name: str | None = pydantic.Field(default=None, description='The name of the unit of the concept')
    unit_type: UnitType | None = pydantic.Field(default=None, description='The type of the unit of the concept')


class ConceptsEndpoint(Endpoint):
    """Endpoint for compensations/concepts operations."""

    endpoint = 'compensations/concepts'

    async def all(self, **kwargs) -> ListApiResponse[Concept]:
        """Get all concepts records."""
        data = await self.api.get_all(self.endpoint, **kwargs)
        return ListApiResponse(model_type=Concept, raw_data=data)

    async def get(self, **kwargs) -> MetaApiResponse[Concept]:
        """Get concepts with pagination metadata."""
        query_params = kwargs.pop('params', {})
        query_params.setdefault('page', 1)
        response = await self.api.get(self.endpoint, params=query_params, **kwargs)
        return MetaApiResponse(model_type=Concept, raw_meta=response['meta'], raw_data=response['data'])

    async def get_by_id(self, concept_id: int | str, **kwargs) -> Concept:
        """Get a specific concept by ID."""
        data = await self.api.get(self.endpoint, concept_id, **kwargs)
        return pydantic.TypeAdapter(Concept).validate_python(data)



import datetime
import typing
from collections.abc import Mapping

import pydantic

from factorialhr._client import Endpoint, ListApiResponse, MetaApiResponse


class SyncableItem(pydantic.BaseModel):
    """Model for integrations_syncable_item."""

    model_config = pydantic.ConfigDict(frozen=True)

    id: int = pydantic.Field(description='Syncable item identifier')
    sync_run_id: int = pydantic.Field(description='Sync run identifier')
    item_type: str = pydantic.Field(description='Type of the syncable item')
    item_id: int = pydantic.Field(description='Identifier of the item')
    status: str = pydantic.Field(description='Status of the syncable item')
    created_at: datetime.datetime = pydantic.Field(description='Creation date')
    updated_at: datetime.datetime = pydantic.Field(description='Last update date')


class SyncableSyncRun(pydantic.BaseModel):
    """Model for integrations_syncable_sync_run."""

    model_config = pydantic.ConfigDict(frozen=True)

    id: int = pydantic.Field(description='Sync run identifier')
    syncable_item_id: int = pydantic.Field(description='Syncable item identifier')
    status: str = pydantic.Field(description='Status of the sync run')
    started_at: datetime.datetime | None = pydantic.Field(default=None, description='Start date')
    completed_at: datetime.datetime | None = pydantic.Field(default=None, description='Completion date')
    created_at: datetime.datetime = pydantic.Field(description='Creation date')
    updated_at: datetime.datetime = pydantic.Field(description='Last update date')


class SyncableItemsEndpoint(Endpoint):
    """Endpoint for integrations/syncable_items operations."""

    endpoint = 'integrations/syncable_items'

    async def all(self, **kwargs) -> ListApiResponse[SyncableItem]:
        """Get all syncable items records."""
        data = await self.api.get_all(self.endpoint, **kwargs)
        return ListApiResponse(model_type=SyncableItem, raw_data=data)

    async def get(self, **kwargs) -> MetaApiResponse[SyncableItem]:
        """Get syncable items with pagination metadata."""
        query_params = kwargs.pop('params', {})
        query_params.setdefault('page', 1)
        response = await self.api.get(self.endpoint, params=query_params, **kwargs)
        return MetaApiResponse(model_type=SyncableItem, raw_meta=response['meta'], raw_data=response['data'])

    async def get_by_id(self, item_id: int | str, **kwargs) -> SyncableItem:
        """Get a specific syncable item by ID."""
        data = await self.api.get(self.endpoint, item_id, **kwargs)
        return pydantic.TypeAdapter(SyncableItem).validate_python(data)


class SyncableSyncRunsEndpoint(Endpoint):
    """Endpoint for integrations/syncable_sync_runs operations."""

    endpoint = 'integrations/syncable_sync_runs'

    async def all(self, **kwargs) -> ListApiResponse[SyncableSyncRun]:
        """Get all syncable sync runs records."""
        data = await self.api.get_all(self.endpoint, **kwargs)
        return ListApiResponse(model_type=SyncableSyncRun, raw_data=data)

    async def get(self, **kwargs) -> MetaApiResponse[SyncableSyncRun]:
        """Get syncable sync runs with pagination metadata."""
        query_params = kwargs.pop('params', {})
        query_params.setdefault('page', 1)
        response = await self.api.get(self.endpoint, params=query_params, **kwargs)
        return MetaApiResponse(model_type=SyncableSyncRun, raw_meta=response['meta'], raw_data=response['data'])

    async def get_by_id(self, run_id: int | str, **kwargs) -> SyncableSyncRun:
        """Get a specific syncable sync run by ID."""
        data = await self.api.get(self.endpoint, run_id, **kwargs)
        return pydantic.TypeAdapter(SyncableSyncRun).validate_python(data)

    async def update(self, run_id: int | str, data: Mapping[str, typing.Any], **kwargs) -> SyncableSyncRun:
        """Update a syncable sync run."""
        response = await self.api.put(self.endpoint, run_id, json=data, **kwargs)
        return pydantic.TypeAdapter(SyncableSyncRun).validate_python(response)



import datetime
import typing
from collections.abc import Mapping, Sequence
from enum import StrEnum

import pydantic

from factorialhr._client import Endpoint, ListApiResponse, MetaApiResponse


class AgreementStatus(StrEnum):
    """Enum for agreement status."""

    PENDING = 'pending'
    SIGNED = 'signed'


class ReviewerStrategy(StrEnum):
    """Enum for reviewer strategy types."""

    SELF = 'self'
    MANAGER = 'manager'
    DIRECT_REPORTS = 'direct_reports'
    PEERS = 'peers'


class EvaluationType(StrEnum):
    """Enum for evaluation types."""

    SELF = 'self'
    MANAGER = 'manager'
    DIRECT_REPORTS = 'direct_reports'
    PEERS = 'peers'


class EvaluationStatus(StrEnum):
    """Enum for evaluation status."""

    PENDING = 'pending'
    PUBLISHED = 'published'


class ProcessStatus(StrEnum):
    """Enum for review process status."""

    DRAFT = 'draft'
    SCHEDULED = 'scheduled'
    ACTIVE = 'active'
    FINISHED = 'finished'


class StartValidationError(StrEnum):
    """Enum for start validation errors."""

    MISSING_DEADLINE = 'missing_deadline'
    INVALID_DEADLINE = 'invalid_deadline'
    MISSING_TITLE = 'missing_title'
    MISSING_TARGET_STRATEGY = 'missing_target_strategy'
    MISSING_TARGET_STRATEGY_MEMBERS = 'missing_target_strategy_members'
    MISSING_REVIEWER_STRATEGY = 'missing_reviewer_strategy'
    MISSING_QUESTIONS = 'missing_questions'
    MISSING_POTENTIAL_REVIEWERS = 'missing_potential_reviewers'
    INVALID_SECTION_WEIGHTS_SUM = 'invalid_section_weights_sum'


class Agreement(pydantic.BaseModel):
    """Model for performance_agreement."""

    model_config = pydantic.ConfigDict(frozen=True)

    id: str = pydantic.Field(description='Action plan ID')
    process_id: int = pydantic.Field(description='Review process ID')
    target_id: str = pydantic.Field(description='Review process target ID')
    signer_id: int | None = pydantic.Field(default=None, description='Manager access ID who signed the action plan')
    reviewer_id: int | None = pydantic.Field(default=None, description='Manager employee ID')
    manager_signed_at: datetime.datetime | None = pydantic.Field(
        default=None,
        description='Date when the manager signed the action plan',
    )
    target_signed_at: datetime.datetime | None = pydantic.Field(
        default=None,
        description='Date when the employee signed the action plan',
    )
    agreement_signed_at: datetime.datetime | None = pydantic.Field(
        default=None,
        description='Date when the action plan was last signed',
    )
    last_modified_at: datetime.datetime | None = pydantic.Field(
        default=None,
        description='Date when the action plan was last modified',
    )
    status: AgreementStatus = pydantic.Field(description='Action plan status')
    locked: bool = pydantic.Field(
        description='When the action plan cannot be edited anymore. Locked when both manager and employee signed it',
    )
    conclusions: Mapping[str, typing.Any] | None = pydantic.Field(
        default=None,
        description='Conclusions of the action plan',
    )
    self_evaluation_id: int | None = pydantic.Field(default=None, description='Self review evaluation ID')
    manager_evaluation_id: int | None = pydantic.Field(default=None, description='Manager review evaluation ID')
    self_comments: Sequence[Mapping[str, typing.Any]] = pydantic.Field(description='Self comments by question')
    manager_comments: Sequence[Mapping[str, typing.Any]] = pydantic.Field(description='Manager comments by question')


class CompanyEmployeeScoreScale(pydantic.BaseModel):
    """Model for performance_company_employee_score_scale."""

    model_config = pydantic.ConfigDict(frozen=True)

    id: int = pydantic.Field(description='Company ID')
    scale_id: int = pydantic.Field(description='Employee score scale ID')
    is_default: bool = pydantic.Field(description='Default employee score scale')


class EmployeeScoreScale(pydantic.BaseModel):
    """Model for performance_employee_score_scale."""

    model_config = pydantic.ConfigDict(frozen=True)

    id: int = pydantic.Field(description='Employee score scale ID')
    scale: Sequence[typing.Any] = pydantic.Field(description='Scale to be used when scoring the employee performance')
    is_default: bool = pydantic.Field(description='Whether this is the default score scale')


class ReviewEvaluationScore(pydantic.BaseModel):
    """Model for performance_review_evaluation_score."""

    model_config = pydantic.ConfigDict(frozen=True)

    id: int = pydantic.Field(description='Review evaluation score identifier')
    review_evaluation_id: int = pydantic.Field(description='Review evaluation identifier')
    score: float = pydantic.Field(description='Evaluation score')
    created_at: datetime.datetime = pydantic.Field(description='Creation date')
    updated_at: datetime.datetime = pydantic.Field(description='Last update date')


class ReviewEvaluation(pydantic.BaseModel):
    """Model for performance_review_evaluation."""

    model_config = pydantic.ConfigDict(frozen=True)

    id: int = pydantic.Field(description='Evaluation ID')
    performance_review_process_id: int | None = pydantic.Field(default=None, description='Review process ID')
    target_access_id: int | None = pydantic.Field(default=None, description='Participant access ID')
    reviewer_access_id: int | None = pydantic.Field(default=None, description='Reviewer access ID')
    evaluation_type: EvaluationType = pydantic.Field(description='Evaluation type')
    published: bool = pydantic.Field(description='Whether the evaluation is published')
    status: EvaluationStatus = pydantic.Field(description='Evaluation status')
    review_process_target_id: str = pydantic.Field(description='Review process target identifier')
    published_at: datetime.datetime | None = pydantic.Field(
        default=None,
        description='Date when the evaluation was published',
    )


class ReviewEvaluationAnswer(pydantic.BaseModel):
    """Model for performance_review_evaluation_answer."""

    model_config = pydantic.ConfigDict(frozen=True)

    id: int = pydantic.Field(description='Review evaluation ID')
    performance_review_evaluation_id: int = pydantic.Field(description='Review evaluation ID')
    answered_questionnaire_with_sections: Mapping[str, typing.Any] = pydantic.Field(
        description='List of questions and their respective answers grouped by section',
    )
    answered_employee_score_questionnaire: Mapping[str, typing.Any] | None = pydantic.Field(
        default=None,
        description='Questionnaire for getting employee score',
    )
    answered_employee_potential_score_questionnaire: Mapping[str, typing.Any] | None = pydantic.Field(
        default=None,
        description='Questionnaire for getting the employee potential score',
    )


class ReviewOwner(pydantic.BaseModel):
    """Model for performance_review_owner."""

    model_config = pydantic.ConfigDict(frozen=True)

    id: int = pydantic.Field(description='Review owner ID')
    access_id: int = pydantic.Field(description='Review owner access ID')
    performance_review_process_id: int = pydantic.Field(description='Review process ID')


class ReviewProcess(pydantic.BaseModel):
    """Model for performance_review_process."""

    model_config = pydantic.ConfigDict(frozen=True)

    id: int = pydantic.Field(description='Review process ID')
    company_id: int = pydantic.Field(description='Company ID')
    name: str | None = pydantic.Field(default=None, description='Review process name')
    description: str | None = pydantic.Field(default=None, description='A brief description of the review process')
    status: ProcessStatus = pydantic.Field(description='Review process status')
    target_strategy: Mapping[str, typing.Any] | None = pydantic.Field(
        default=None,
        description=(
            'Condition that defines the employees that will be evaluated (participants). '
            'Calculated when the review process starts'
        ),
    )
    reviewer_strategies: Sequence[str] | None = pydantic.Field(
        default=None,
        description=(
            'Review types that will be assigned to the review process. '
            "It'll be used to create the evaluations when the process starts"
        ),
    )
    starts_at: datetime.datetime | None = pydantic.Field(
        default=None,
        description='Date when the review process should start',
    )
    ends_at: datetime.datetime | None = pydantic.Field(
        default=None,
        description='Date when the review process should end',
    )
    start_validation_errors: Sequence[StartValidationError] = pydantic.Field(
        description='Missing or invalid information to be able to start the review process',
    )
    archived: bool = pydantic.Field(description='Whether the review process is archived or not')
    agreements_configuration: Mapping[str, typing.Any] = pydantic.Field(
        description='Action plans help track goal progress, and facilitate performance review discussions',
    )
    competencies_assessments_configuration: Mapping[str, typing.Any] = pydantic.Field(
        description=(
            'Assess employees based on their assigned competencies through both manager and self-reviews. '
            'Ensure roles with designated competencies are properly set up'
        ),
    )
    last_bulk_reminder: datetime.datetime | None = pydantic.Field(
        default=None,
        description='Date when the last bulk reminder was sent',
    )
    cycle_id: str | None = pydantic.Field(default=None, description='Performance cycle ID')


class ReviewProcessCustomTemplate(pydantic.BaseModel):
    """Model for performance_review_process_custom_template."""

    model_config = pydantic.ConfigDict(frozen=True)

    id: int = pydantic.Field(description='Review process template ID')
    author_id: int | None = pydantic.Field(default=None, description='Author of the custom template')
    company_id: int = pydantic.Field(description='Company ID')
    name: str = pydantic.Field(description='Review process name')
    description: str | None = pydantic.Field(default=None, description='A brief description of the review process')
    template_description: str | None = pydantic.Field(
        default=None,
        description='A brief description of the review process template',
    )
    target_strategy: Mapping[str, typing.Any] | None = pydantic.Field(
        default=None,
        description=(
            'Condition that defines the employees that will be evaluated (participants). '
            'Calculated when the review process starts'
        ),
    )
    reviewer_strategies: Sequence[str] | None = pydantic.Field(
        default=None,
        description=(
            'Review types that will be assigned to the review process. '
            "It'll be used to create the evaluations when the process starts"
        ),
    )
    agreements_enabled: bool | None = pydantic.Field(
        default=None,
        description='Action plans help track goal progress, and facilitate performance review discussions',
    )
    employee_potential_score_enabled: bool | None = pydantic.Field(
        default=None,
        description=(
            "Include one question at the end of the review to rate participants' potential. "
            'This rating will be reflected in the 9 box grid'
        ),
    )
    competencies_assessments_enabled: bool | None = pydantic.Field(
        default=None,
        description=(
            'Assess employees based on their assigned competencies through both manager and self-reviews. '
            'Ensure roles with designated competencies are properly set up'
        ),
    )
    visibility_settings: Mapping[str, typing.Any] | None = pydantic.Field(
        default=None,
        description='Visibility settings for the custom template',
    )
    created_at: datetime.datetime = pydantic.Field(description='Creation date of the template')


class ReviewProcessEstimatedTarget(pydantic.BaseModel):
    """Model for performance_review_process_estimated_target."""

    model_config = pydantic.ConfigDict(frozen=True)

    id: str = pydantic.Field(description='Process target ID')
    performance_review_process_id: int = pydantic.Field(description='Review process ID')
    access_id: int = pydantic.Field(description='Access ID')
    employee_id: int = pydantic.Field(description='Employee ID')


class ReviewProcessTarget(pydantic.BaseModel):
    """Model for performance_review_process_target."""

    model_config = pydantic.ConfigDict(frozen=True)

    id: str = pydantic.Field(description='Review process target ID')
    access_id: int = pydantic.Field(description='Participant access ID')
    performance_review_process_id: int = pydantic.Field(description='Review process ID')
    materialized_process_target_id: int = pydantic.Field(description='Materialized process target ID')


class ReviewQuestionnairesByStrategy(pydantic.BaseModel):
    """Model for performance_review_questionnaires_by_strategy."""

    model_config = pydantic.ConfigDict(frozen=True)

    id: int = pydantic.Field(description='Review process ID')
    performance_review_process_id: int = pydantic.Field(description='Review process ID')
    default_rating_scale: Sequence[typing.Any] = pydantic.Field(description='Scoring range used in rating questions')
    self_questionnaire: Mapping[str, typing.Any] | None = pydantic.Field(
        default=None,
        description='Questionnaire for self evaluation',
    )
    manager_questionnaire: Mapping[str, typing.Any] | None = pydantic.Field(
        default=None,
        description='Questionnaire for manager evaluation',
    )
    direct_report_questionnaire: Mapping[str, typing.Any] | None = pydantic.Field(
        default=None,
        description='Questionnaire for direct report evaluation',
    )
    peers_questionnaire: Mapping[str, typing.Any] | None = pydantic.Field(
        default=None,
        description='Questionnaire for peers evaluation',
    )
    employee_score_self_questionnaire: Mapping[str, typing.Any] | None = pydantic.Field(
        default=None,
        description='Questionnaire included in the end of self evaluation to evaluate the employee performance',
    )
    employee_score_manager_questionnaire: Mapping[str, typing.Any] | None = pydantic.Field(
        default=None,
        description='Questionnaire included in the end of manager evaluation to evaluate the employee performance',
    )
    employee_potential_score_manager_questionnaire: Mapping[str, typing.Any] | None = pydantic.Field(
        default=None,
        description='Questionnaire included in the end of manager evaluation to evaluate the employee potential',
    )


class ReviewVisibilitySetting(pydantic.BaseModel):
    """Model for performance_review_visibility_setting."""

    model_config = pydantic.ConfigDict(frozen=True)

    id: int = pydantic.Field(description='Review process ID')
    performance_review_process_id: int = pydantic.Field(description='Review process ID')
    restrict_answers_visibility_to_reportees: bool = pydantic.Field(
        description="Employees don't have access to their results when enabled",
    )
    early_access_to_answers_for_managers: bool = pydantic.Field(
        description='Managers can access the results of their reports before deadline when enabled',
    )
    anonymous_peer_evaluation_for_target: bool = pydantic.Field(
        description="Peer evaluations are anonymous when enabled, so employees don't know who reviewed them",
    )


class TargetManager(pydantic.BaseModel):
    """Model for performance_target_manager."""

    model_config = pydantic.ConfigDict(frozen=True)

    id: str = pydantic.Field(description='Manager employee ID')
    performance_review_process_id: int = pydantic.Field(description='Review process ID')
    manager_access_id: int = pydantic.Field(description='Manager access ID')
    manager_full_name: str = pydantic.Field(description='Manager full name')


class AgreementsEndpoint(Endpoint):
    """Endpoint for performance/agreements operations."""

    endpoint = 'performance/agreements'

    async def all(self, **kwargs) -> ListApiResponse[Agreement]:
        """Get all agreements."""
        data = await self.api.get_all(self.endpoint, **kwargs)
        return ListApiResponse(model_type=Agreement, raw_data=data)

    async def get(self, **kwargs) -> MetaApiResponse[Agreement]:
        """Get agreements with pagination metadata."""
        query_params = kwargs.pop('params', {})
        query_params.setdefault('page', 1)
        response = await self.api.get(self.endpoint, params=query_params, **kwargs)
        return MetaApiResponse(model_type=Agreement, raw_meta=response['meta'], raw_data=response['data'])

    async def get_by_id(self, agreement_id: int | str, **kwargs) -> Agreement:
        """Get a specific agreement by ID."""
        data = await self.api.get(self.endpoint, agreement_id, **kwargs)
        return pydantic.TypeAdapter(Agreement).validate_python(data)

    async def bulk_initiate(self, data: Mapping[str, typing.Any], **kwargs) -> Sequence[Agreement]:
        """Initiate action plans for all direct reports in a review process."""
        response = await self.api.post(self.endpoint, 'bulk_initiate', json=data, **kwargs)
        return [pydantic.TypeAdapter(Agreement).validate_python(item) for item in response]

    async def initiate(self, data: Mapping[str, typing.Any], **kwargs) -> Agreement:
        """Initiate an action plan for a specific review process target."""
        response = await self.api.post(self.endpoint, 'initiate', json=data, **kwargs)
        return pydantic.TypeAdapter(Agreement).validate_python(response)


class CompanyEmployeeScoreScalesEndpoint(Endpoint):
    """Endpoint for performance/company_employee_score_scales operations."""

    endpoint = 'performance/company_employee_score_scales'

    async def all(self, **kwargs) -> ListApiResponse[CompanyEmployeeScoreScale]:
        """Get all company employee score scales."""
        data = await self.api.get_all(self.endpoint, **kwargs)
        return ListApiResponse(model_type=CompanyEmployeeScoreScale, raw_data=data)

    async def get(self, **kwargs) -> MetaApiResponse[CompanyEmployeeScoreScale]:
        """Get company employee score scales with pagination metadata."""
        query_params = kwargs.pop('params', {})
        query_params.setdefault('page', 1)
        response = await self.api.get(self.endpoint, params=query_params, **kwargs)
        return MetaApiResponse(
            model_type=CompanyEmployeeScoreScale,
            raw_meta=response['meta'],
            raw_data=response['data'],
        )

    async def get_by_id(self, scale_id: int | str, **kwargs) -> CompanyEmployeeScoreScale:
        """Get a specific company employee score scale by ID."""
        data = await self.api.get(self.endpoint, scale_id, **kwargs)
        return pydantic.TypeAdapter(CompanyEmployeeScoreScale).validate_python(data)

    async def set_scale(self, data: Mapping[str, typing.Any], **kwargs) -> CompanyEmployeeScoreScale:
        """Set the predefined employee score scale for the company."""
        response = await self.api.post(self.endpoint, 'set', json=data, **kwargs)
        return pydantic.TypeAdapter(CompanyEmployeeScoreScale).validate_python(response)


class EmployeeScoreScalesEndpoint(Endpoint):
    """Endpoint for performance/employee_score_scales operations."""

    endpoint = 'performance/employee_score_scales'

    async def all(self, **kwargs) -> ListApiResponse[EmployeeScoreScale]:
        """Get all employee score scales."""
        data = await self.api.get_all(self.endpoint, **kwargs)
        return ListApiResponse(model_type=EmployeeScoreScale, raw_data=data)

    async def get(self, **kwargs) -> MetaApiResponse[EmployeeScoreScale]:
        """Get employee score scales with pagination metadata."""
        query_params = kwargs.pop('params', {})
        query_params.setdefault('page', 1)
        response = await self.api.get(self.endpoint, params=query_params, **kwargs)
        return MetaApiResponse(model_type=EmployeeScoreScale, raw_meta=response['meta'], raw_data=response['data'])

    async def get_by_id(self, scale_id: int | str, **kwargs) -> EmployeeScoreScale:
        """Get a specific employee score scale by ID."""
        data = await self.api.get(self.endpoint, scale_id, **kwargs)
        return pydantic.TypeAdapter(EmployeeScoreScale).validate_python(data)


class ReviewEvaluationScoresEndpoint(Endpoint):
    """Endpoint for performance/review_evaluation_scores operations."""

    endpoint = 'performance/review_evaluation_scores'

    async def all(self, **kwargs) -> ListApiResponse[ReviewEvaluationScore]:
        """Get all review evaluation scores."""
        data = await self.api.get_all(self.endpoint, **kwargs)
        return ListApiResponse(model_type=ReviewEvaluationScore, raw_data=data)

    async def get(self, **kwargs) -> MetaApiResponse[ReviewEvaluationScore]:
        """Get review evaluation scores with pagination metadata."""
        query_params = kwargs.pop('params', {})
        query_params.setdefault('page', 1)
        response = await self.api.get(self.endpoint, params=query_params, **kwargs)
        return MetaApiResponse(model_type=ReviewEvaluationScore, raw_meta=response['meta'], raw_data=response['data'])

    async def get_by_id(self, score_id: int | str, **kwargs) -> ReviewEvaluationScore:
        """Get a specific review evaluation score by ID."""
        data = await self.api.get(self.endpoint, score_id, **kwargs)
        return pydantic.TypeAdapter(ReviewEvaluationScore).validate_python(data)


class ReviewEvaluationsEndpoint(Endpoint):
    """Endpoint for performance/review_evaluations operations."""

    endpoint = 'performance/review_evaluations'

    async def all(self, **kwargs) -> ListApiResponse[ReviewEvaluation]:
        """Get all review evaluations."""
        data = await self.api.get_all(self.endpoint, **kwargs)
        return ListApiResponse(model_type=ReviewEvaluation, raw_data=data)

    async def get(self, **kwargs) -> MetaApiResponse[ReviewEvaluation]:
        """Get review evaluations with pagination metadata."""
        query_params = kwargs.pop('params', {})
        query_params.setdefault('page', 1)
        response = await self.api.get(self.endpoint, params=query_params, **kwargs)
        return MetaApiResponse(model_type=ReviewEvaluation, raw_meta=response['meta'], raw_data=response['data'])

    async def get_by_id(self, evaluation_id: int | str, **kwargs) -> ReviewEvaluation:
        """Get a specific review evaluation by ID."""
        data = await self.api.get(self.endpoint, evaluation_id, **kwargs)
        return pydantic.TypeAdapter(ReviewEvaluation).validate_python(data)

    async def replace_reviewer(self, data: Mapping[str, typing.Any], **kwargs) -> ReviewEvaluation:
        """Replace a reviewer for an evaluation."""
        response = await self.api.post(self.endpoint, 'replace_reviewer', json=data, **kwargs)
        return pydantic.TypeAdapter(ReviewEvaluation).validate_python(response)


class ReviewEvaluationAnswersEndpoint(Endpoint):
    """Endpoint for performance/review_evaluation_answers operations."""

    endpoint = 'performance/review_evaluation_answers'

    async def all(self, **kwargs) -> ListApiResponse[ReviewEvaluationAnswer]:
        """Get all review evaluation answers."""
        data = await self.api.get_all(self.endpoint, **kwargs)
        return ListApiResponse(model_type=ReviewEvaluationAnswer, raw_data=data)

    async def get(self, **kwargs) -> MetaApiResponse[ReviewEvaluationAnswer]:
        """Get review evaluation answers with pagination metadata."""
        query_params = kwargs.pop('params', {})
        query_params.setdefault('page', 1)
        response = await self.api.get(self.endpoint, params=query_params, **kwargs)
        return MetaApiResponse(model_type=ReviewEvaluationAnswer, raw_meta=response['meta'], raw_data=response['data'])


class ReviewOwnersEndpoint(Endpoint):
    """Endpoint for performance/review_owners operations."""

    endpoint = 'performance/review_owners'

    async def all(self, **kwargs) -> ListApiResponse[ReviewOwner]:
        """Get all review owners."""
        data = await self.api.get_all(self.endpoint, **kwargs)
        return ListApiResponse(model_type=ReviewOwner, raw_data=data)

    async def get(self, **kwargs) -> MetaApiResponse[ReviewOwner]:
        """Get review owners with pagination metadata."""
        query_params = kwargs.pop('params', {})
        query_params.setdefault('page', 1)
        response = await self.api.get(self.endpoint, params=query_params, **kwargs)
        return MetaApiResponse(model_type=ReviewOwner, raw_meta=response['meta'], raw_data=response['data'])

    async def delete(self, owner_id: int | str, **kwargs) -> ReviewOwner:
        """Delete a review owner."""
        response = await self.api.delete(self.endpoint, owner_id, **kwargs)
        return pydantic.TypeAdapter(ReviewOwner).validate_python(response)

    async def bulk_create(self, data: Mapping[str, typing.Any], **kwargs) -> Sequence[ReviewOwner]:
        """Add multiple owners to a review process."""
        response = await self.api.post(self.endpoint, 'bulk_create', json=data, **kwargs)
        return [pydantic.TypeAdapter(ReviewOwner).validate_python(item) for item in response]


class ReviewProcessesEndpoint(Endpoint):
    """Endpoint for performance/review_processes operations."""

    endpoint = 'performance/review_processes'

    async def all(self, **kwargs) -> ListApiResponse[ReviewProcess]:
        """Get all review processes."""
        data = await self.api.get_all(self.endpoint, **kwargs)
        return ListApiResponse(model_type=ReviewProcess, raw_data=data)

    async def get(self, **kwargs) -> MetaApiResponse[ReviewProcess]:
        """Get review processes with pagination metadata."""
        query_params = kwargs.pop('params', {})
        query_params.setdefault('page', 1)
        response = await self.api.get(self.endpoint, params=query_params, **kwargs)
        return MetaApiResponse(model_type=ReviewProcess, raw_meta=response['meta'], raw_data=response['data'])

    async def get_by_id(self, process_id: int | str, **kwargs) -> ReviewProcess:
        """Get a specific review process by ID."""
        data = await self.api.get(self.endpoint, process_id, **kwargs)
        return pydantic.TypeAdapter(ReviewProcess).validate_python(data)

    async def create(self, data: Mapping[str, typing.Any], **kwargs) -> ReviewProcess:
        """Create a new review process."""
        response = await self.api.post(self.endpoint, json=data, **kwargs)
        return pydantic.TypeAdapter(ReviewProcess).validate_python(response)

    async def delete(self, process_id: int | str, **kwargs) -> ReviewProcess:
        """Delete a review process."""
        response = await self.api.delete(self.endpoint, process_id, **kwargs)
        return pydantic.TypeAdapter(ReviewProcess).validate_python(response)

    async def create_from_template(self, data: Mapping[str, typing.Any], **kwargs) -> ReviewProcess:
        """Create a new review process from a template."""
        response = await self.api.post(self.endpoint, 'create_from_template', json=data, **kwargs)
        return pydantic.TypeAdapter(ReviewProcess).validate_python(response)

    async def duplicate(self, data: Mapping[str, typing.Any], **kwargs) -> ReviewProcess:
        """Duplicate an existing review process."""
        response = await self.api.post(self.endpoint, 'duplicate', json=data, **kwargs)
        return pydantic.TypeAdapter(ReviewProcess).validate_python(response)

    async def remind_in_bulk(self, data: Mapping[str, typing.Any], **kwargs) -> ReviewProcess:
        """Send bulk reminders to reviewers."""
        response = await self.api.post(self.endpoint, 'remind_in_bulk', json=data, **kwargs)
        return pydantic.TypeAdapter(ReviewProcess).validate_python(response)

    async def remove_schedule(self, data: Mapping[str, typing.Any], **kwargs) -> ReviewProcess:
        """Remove the schedule job for a review process."""
        response = await self.api.post(self.endpoint, 'remove_schedule', json=data, **kwargs)
        return pydantic.TypeAdapter(ReviewProcess).validate_python(response)

    async def reopen(self, data: Mapping[str, typing.Any], **kwargs) -> ReviewProcess:
        """Reopen a finished review process."""
        response = await self.api.post(self.endpoint, 'reopen', json=data, **kwargs)
        return pydantic.TypeAdapter(ReviewProcess).validate_python(response)

    async def schedule(self, data: Mapping[str, typing.Any], **kwargs) -> ReviewProcess:
        """Schedule a review process to start at a given date."""
        response = await self.api.post(self.endpoint, 'schedule', json=data, **kwargs)
        return pydantic.TypeAdapter(ReviewProcess).validate_python(response)

    async def start(self, data: Mapping[str, typing.Any], **kwargs) -> ReviewProcess:
        """Start a review process."""
        response = await self.api.post(self.endpoint, 'start', json=data, **kwargs)
        return pydantic.TypeAdapter(ReviewProcess).validate_python(response)

    async def stop(self, data: Mapping[str, typing.Any], **kwargs) -> ReviewProcess:
        """Stop a review process."""
        response = await self.api.post(self.endpoint, 'stop', json=data, **kwargs)
        return pydantic.TypeAdapter(ReviewProcess).validate_python(response)

    async def toggle_archive(self, data: Mapping[str, typing.Any], **kwargs) -> ReviewProcess:
        """Archive or unarchive a review process."""
        response = await self.api.post(self.endpoint, 'toggle_archive', json=data, **kwargs)
        return pydantic.TypeAdapter(ReviewProcess).validate_python(response)

    async def update_agreements_configuration(self, data: Mapping[str, typing.Any], **kwargs) -> ReviewProcess:
        """Enable or disable action plans in a review process."""
        response = await self.api.post(self.endpoint, 'update_agreements_configuration', json=data, **kwargs)
        return pydantic.TypeAdapter(ReviewProcess).validate_python(response)

    async def update_basic_info(self, data: Mapping[str, typing.Any], **kwargs) -> ReviewProcess:
        """Update the basic information of a review process."""
        response = await self.api.post(self.endpoint, 'update_basic_info', json=data, **kwargs)
        return pydantic.TypeAdapter(ReviewProcess).validate_python(response)

    async def update_competencies_assessments_configuration(
        self,
        data: Mapping[str, typing.Any],
        **kwargs,
    ) -> ReviewProcess:
        """Update competencies assessments configuration."""
        response = await self.api.post(
            self.endpoint,
            'update_competencies_assessments_configuration',
            json=data,
            **kwargs,
        )
        return pydantic.TypeAdapter(ReviewProcess).validate_python(response)

    async def update_deadline(self, data: Mapping[str, typing.Any], **kwargs) -> ReviewProcess:
        """Update the deadline of a review process."""
        response = await self.api.post(self.endpoint, 'update_deadline', json=data, **kwargs)
        return pydantic.TypeAdapter(ReviewProcess).validate_python(response)

    async def update_employee_score_configuration(self, data: Mapping[str, typing.Any], **kwargs) -> ReviewProcess:
        """Update employee score configuration."""
        response = await self.api.post(self.endpoint, 'update_employee_score_configuration', json=data, **kwargs)
        return pydantic.TypeAdapter(ReviewProcess).validate_python(response)

    async def update_reviewer_strategies(self, data: Mapping[str, typing.Any], **kwargs) -> ReviewProcess:
        """Update the review types of a review process."""
        response = await self.api.post(self.endpoint, 'update_reviewer_strategies', json=data, **kwargs)
        return pydantic.TypeAdapter(ReviewProcess).validate_python(response)

    async def update_schedule(self, data: Mapping[str, typing.Any], **kwargs) -> ReviewProcess:
        """Update the starting date of a scheduled review process."""
        response = await self.api.post(self.endpoint, 'update_schedule', json=data, **kwargs)
        return pydantic.TypeAdapter(ReviewProcess).validate_python(response)

    async def update_target_strategy(self, data: Mapping[str, typing.Any], **kwargs) -> ReviewProcess:
        """Update the criteria for calculating the participants of a review process."""
        response = await self.api.post(self.endpoint, 'update_target_strategy', json=data, **kwargs)
        return pydantic.TypeAdapter(ReviewProcess).validate_python(response)


class ReviewProcessCustomTemplatesEndpoint(Endpoint):
    """Endpoint for performance/review_process_custom_templates operations."""

    endpoint = 'performance/review_process_custom_templates'

    async def all(self, **kwargs) -> ListApiResponse[ReviewProcessCustomTemplate]:
        """Get all review process custom templates."""
        data = await self.api.get_all(self.endpoint, **kwargs)
        return ListApiResponse(model_type=ReviewProcessCustomTemplate, raw_data=data)

    async def get(self, **kwargs) -> MetaApiResponse[ReviewProcessCustomTemplate]:
        """Get review process custom templates with pagination metadata."""
        query_params = kwargs.pop('params', {})
        query_params.setdefault('page', 1)
        response = await self.api.get(self.endpoint, params=query_params, **kwargs)
        return MetaApiResponse(
            model_type=ReviewProcessCustomTemplate,
            raw_meta=response['meta'],
            raw_data=response['data'],
        )

    async def get_by_id(self, template_id: int | str, **kwargs) -> ReviewProcessCustomTemplate:
        """Get a specific review process custom template by ID."""
        data = await self.api.get(self.endpoint, template_id, **kwargs)
        return pydantic.TypeAdapter(ReviewProcessCustomTemplate).validate_python(data)


class ReviewProcessEstimatedTargetsEndpoint(Endpoint):
    """Endpoint for performance/review_process_estimated_targets operations."""

    endpoint = 'performance/review_process_estimated_targets'

    async def all(self, **kwargs) -> ListApiResponse[ReviewProcessEstimatedTarget]:
        """Get all review process estimated targets."""
        data = await self.api.get_all(self.endpoint, **kwargs)
        return ListApiResponse(model_type=ReviewProcessEstimatedTarget, raw_data=data)

    async def get(self, **kwargs) -> MetaApiResponse[ReviewProcessEstimatedTarget]:
        """Get review process estimated targets with pagination metadata."""
        query_params = kwargs.pop('params', {})
        query_params.setdefault('page', 1)
        response = await self.api.get(self.endpoint, params=query_params, **kwargs)
        return MetaApiResponse(
            model_type=ReviewProcessEstimatedTarget,
            raw_meta=response['meta'],
            raw_data=response['data'],
        )


class ReviewProcessTargetsEndpoint(Endpoint):
    """Endpoint for performance/review_process_targets operations."""

    endpoint = 'performance/review_process_targets'

    async def all(self, **kwargs) -> ListApiResponse[ReviewProcessTarget]:
        """Get all review process targets."""
        data = await self.api.get_all(self.endpoint, **kwargs)
        return ListApiResponse(model_type=ReviewProcessTarget, raw_data=data)

    async def get(self, **kwargs) -> MetaApiResponse[ReviewProcessTarget]:
        """Get review process targets with pagination metadata."""
        query_params = kwargs.pop('params', {})
        query_params.setdefault('page', 1)
        response = await self.api.get(self.endpoint, params=query_params, **kwargs)
        return MetaApiResponse(model_type=ReviewProcessTarget, raw_meta=response['meta'], raw_data=response['data'])

    async def get_by_id(self, target_id: str, **kwargs) -> ReviewProcessTarget:
        """Get a specific review process target by ID."""
        data = await self.api.get(self.endpoint, target_id, **kwargs)
        return pydantic.TypeAdapter(ReviewProcessTarget).validate_python(data)

    async def delete(self, target_id: str, **kwargs) -> ReviewProcessTarget:
        """Delete a participant from the active review process."""
        response = await self.api.delete(self.endpoint, target_id, **kwargs)
        return pydantic.TypeAdapter(ReviewProcessTarget).validate_python(response)

    async def add_peers(self, data: Mapping[str, typing.Any], **kwargs) -> ReviewProcessTarget:
        """Assign peers to evaluate a specific participant."""
        response = await self.api.post(self.endpoint, 'add_peers', json=data, **kwargs)
        return pydantic.TypeAdapter(ReviewProcessTarget).validate_python(response)

    async def bulk_create(self, data: Mapping[str, typing.Any], **kwargs) -> Sequence[ReviewProcessTarget]:
        """Add multiple participants to the active review process."""
        response = await self.api.post(self.endpoint, 'bulk_create', json=data, **kwargs)
        return [pydantic.TypeAdapter(ReviewProcessTarget).validate_python(item) for item in response]

    async def remove_peer_evaluations(self, data: Mapping[str, typing.Any], **kwargs) -> ReviewProcessTarget:
        """Remove peers and their evaluations from a specific participant."""
        response = await self.api.post(self.endpoint, 'remove_peer_evaluations', json=data, **kwargs)
        return pydantic.TypeAdapter(ReviewProcessTarget).validate_python(response)


class ReviewQuestionnaireByStrategiesEndpoint(Endpoint):
    """Endpoint for performance/review_questionnaire_by_strategies operations."""

    endpoint = 'performance/review_questionnaire_by_strategies'

    async def all(self, **kwargs) -> ListApiResponse[ReviewQuestionnairesByStrategy]:
        """Get all review questionnaires by strategies."""
        data = await self.api.get_all(self.endpoint, **kwargs)
        return ListApiResponse(model_type=ReviewQuestionnairesByStrategy, raw_data=data)

    async def get(self, **kwargs) -> MetaApiResponse[ReviewQuestionnairesByStrategy]:
        """Get review questionnaires by strategies with pagination metadata."""
        query_params = kwargs.pop('params', {})
        query_params.setdefault('page', 1)
        response = await self.api.get(self.endpoint, params=query_params, **kwargs)
        return MetaApiResponse(
            model_type=ReviewQuestionnairesByStrategy,
            raw_meta=response['meta'],
            raw_data=response['data'],
        )

    async def get_by_id(self, questionnaire_id: int | str, **kwargs) -> ReviewQuestionnairesByStrategy:
        """Get a specific review questionnaire by strategy by ID."""
        data = await self.api.get(self.endpoint, questionnaire_id, **kwargs)
        return pydantic.TypeAdapter(ReviewQuestionnairesByStrategy).validate_python(data)

    async def update_default_rating_scale(
        self,
        data: Mapping[str, typing.Any],
        **kwargs,
    ) -> ReviewQuestionnairesByStrategy:
        """Update the scoring range used in rating questions for all reviewer strategies."""
        response = await self.api.post(self.endpoint, 'update_default_rating_scale', json=data, **kwargs)
        return pydantic.TypeAdapter(ReviewQuestionnairesByStrategy).validate_python(response)

    async def update_questionnaire_for_strategy(
        self,
        data: Mapping[str, typing.Any],
        **kwargs,
    ) -> ReviewQuestionnairesByStrategy:
        """Update the review process questionnaire for a specific reviewer strategy."""
        response = await self.api.post(
            self.endpoint,
            'update_questionnaire_for_strategy',
            json=data,
            **kwargs,
        )
        return pydantic.TypeAdapter(ReviewQuestionnairesByStrategy).validate_python(response)


class ReviewVisibilitySettingsEndpoint(Endpoint):
    """Endpoint for performance/review_visibility_settings operations."""

    endpoint = 'performance/review_visibility_settings'

    async def all(self, **kwargs) -> ListApiResponse[ReviewVisibilitySetting]:
        """Get all review visibility settings."""
        data = await self.api.get_all(self.endpoint, **kwargs)
        return ListApiResponse(model_type=ReviewVisibilitySetting, raw_data=data)

    async def get(self, **kwargs) -> MetaApiResponse[ReviewVisibilitySetting]:
        """Get review visibility settings with pagination metadata."""
        query_params = kwargs.pop('params', {})
        query_params.setdefault('page', 1)
        response = await self.api.get(self.endpoint, params=query_params, **kwargs)
        return MetaApiResponse(model_type=ReviewVisibilitySetting, raw_meta=response['meta'], raw_data=response['data'])

    async def update(self, setting_id: int | str, data: Mapping[str, typing.Any], **kwargs) -> ReviewVisibilitySetting:
        """Update the visibility settings of a review process."""
        response = await self.api.put(self.endpoint, setting_id, json=data, **kwargs)
        return pydantic.TypeAdapter(ReviewVisibilitySetting).validate_python(response)


class TargetManagersEndpoint(Endpoint):
    """Endpoint for performance/target_managers operations."""

    endpoint = 'performance/target_managers'

    async def all(self, **kwargs) -> ListApiResponse[TargetManager]:
        """Get all target managers."""
        data = await self.api.get_all(self.endpoint, **kwargs)
        return ListApiResponse(model_type=TargetManager, raw_data=data)

    async def get(self, **kwargs) -> MetaApiResponse[TargetManager]:
        """Get target managers with pagination metadata."""
        query_params = kwargs.pop('params', {})
        query_params.setdefault('page', 1)
        response = await self.api.get(self.endpoint, params=query_params, **kwargs)
        return MetaApiResponse(model_type=TargetManager, raw_meta=response['meta'], raw_data=response['data'])

    async def get_by_id(self, manager_id: int | str, **kwargs) -> TargetManager:
        """Get a specific target manager by ID."""
        data = await self.api.get(self.endpoint, manager_id, **kwargs)
        return pydantic.TypeAdapter(TargetManager).validate_python(data)



import datetime
import typing
from collections.abc import Mapping, Sequence

import pydantic

from factorialhr._client import Endpoint, ListApiResponse, MetaApiResponse


class WorkScheduleDayConfiguration(pydantic.BaseModel):
    """Model for work_schedule_day_configuration."""

    model_config = pydantic.ConfigDict(frozen=True)

    id: int = pydantic.Field(description='Day configuration ID')
    overlap_period_id: int = pydantic.Field(description='Overlap period ID')
    weekday: str = pydantic.Field(description='Day of the week')
    start_at: datetime.date | None = pydantic.Field(default=None, description='Start time')
    duration_in_seconds: int = pydantic.Field(description='Duration in seconds')


class WorkScheduleOverlapPeriod(pydantic.BaseModel):
    """Model for work_schedule_overlap_period."""

    model_config = pydantic.ConfigDict(frozen=True)

    id: int = pydantic.Field(description='Overlap period ID')
    default: bool = pydantic.Field(description='Whether this is the default period')
    schedule_id: int = pydantic.Field(description='Schedule ID')
    start_month: int = pydantic.Field(description='Start month')
    start_day: int = pydantic.Field(description='Start day')
    end_month: int = pydantic.Field(description='End month')
    end_day: int = pydantic.Field(description='End day')
    schedule_type: str = pydantic.Field(description='Type of schedule')


class WorkScheduleSchedule(pydantic.BaseModel):
    """Model for work_schedule_schedule."""

    model_config = pydantic.ConfigDict(frozen=True)

    id: int = pydantic.Field(description='Schedule ID')
    name: str = pydantic.Field(description='Schedule name')
    archived_at: datetime.datetime | None = pydantic.Field(default=None, description='Archive date')
    company_id: int = pydantic.Field(description='Company ID')
    created_at: datetime.datetime = pydantic.Field(description='Creation date')
    updated_at: datetime.datetime = pydantic.Field(description='Last update date')
    employee_ids: Sequence[int] = pydantic.Field(description='List of employee IDs')
    periods: Sequence[typing.Any] = pydantic.Field(description='Schedule periods')


class DayConfigurationEndpoint(Endpoint):
    """Endpoint for work_schedule/day_configurations operations."""

    endpoint = 'work_schedule/day_configurations'

    async def all(self, **kwargs) -> ListApiResponse[WorkScheduleDayConfiguration]:
        """Get all day configuration records."""
        data = await self.api.get_all(self.endpoint, **kwargs)
        return ListApiResponse(raw_data=data, model_type=WorkScheduleDayConfiguration)

    async def get(self, **kwargs) -> MetaApiResponse[WorkScheduleDayConfiguration]:
        """Get day configurations with pagination metadata."""
        query_params = kwargs.pop('params', {})
        query_params.setdefault('page', 1)
        response = await self.api.get(self.endpoint, params=query_params, **kwargs)
        return MetaApiResponse(
            raw_meta=response['meta'],
            raw_data=response['data'],
            model_type=WorkScheduleDayConfiguration,
        )

    async def get_by_id(self, day_config_id: int | str, **kwargs) -> WorkScheduleDayConfiguration:
        """Get a specific day configuration by ID."""
        data = await self.api.get(self.endpoint, day_config_id, **kwargs)
        return pydantic.TypeAdapter(WorkScheduleDayConfiguration).validate_python(data)

    async def bulk_cud(self, data: Mapping[str, typing.Any], **kwargs) -> WorkScheduleDayConfiguration:
        """Bulk create/update/delete day configurations."""
        response = await self.api.post(self.endpoint, 'bulk_cud', json=data, **kwargs)
        return pydantic.TypeAdapter(WorkScheduleDayConfiguration).validate_python(response)


class OverlapPeriodEndpoint(Endpoint):
    """Endpoint for work_schedule/overlap_periods operations."""

    endpoint = 'work_schedule/overlap_periods'

    async def all(self, **kwargs) -> ListApiResponse[WorkScheduleOverlapPeriod]:
        """Get all overlap period records."""
        data = await self.api.get_all(self.endpoint, **kwargs)
        return ListApiResponse(raw_data=data, model_type=WorkScheduleOverlapPeriod)

    async def get(self, **kwargs) -> MetaApiResponse[WorkScheduleOverlapPeriod]:
        """Get overlap periods with pagination metadata."""
        query_params = kwargs.pop('params', {})
        query_params.setdefault('page', 1)
        response = await self.api.get(self.endpoint, params=query_params, **kwargs)
        return MetaApiResponse(
            raw_meta=response['meta'],
            raw_data=response['data'],
            model_type=WorkScheduleOverlapPeriod,
        )

    async def get_by_id(self, overlap_period_id: int | str, **kwargs) -> WorkScheduleOverlapPeriod:
        """Get a specific overlap period by ID."""
        data = await self.api.get(self.endpoint, overlap_period_id, **kwargs)
        return pydantic.TypeAdapter(WorkScheduleOverlapPeriod).validate_python(data)

    async def create(self, data: Mapping[str, typing.Any], **kwargs) -> WorkScheduleOverlapPeriod:
        """Create a new overlap period."""
        response = await self.api.post(self.endpoint, json=data, **kwargs)
        return pydantic.TypeAdapter(WorkScheduleOverlapPeriod).validate_python(response)

    async def update(
        self,
        overlap_period_id: int | str,
        data: Mapping[str, typing.Any],
        **kwargs,
    ) -> WorkScheduleOverlapPeriod:
        """Update an overlap period."""
        response = await self.api.put(self.endpoint, overlap_period_id, json=data, **kwargs)
        return pydantic.TypeAdapter(WorkScheduleOverlapPeriod).validate_python(response)

    async def delete(self, overlap_period_id: int | str, **kwargs) -> WorkScheduleOverlapPeriod:
        """Delete an overlap period."""
        response = await self.api.delete(self.endpoint, overlap_period_id, **kwargs)
        return pydantic.TypeAdapter(WorkScheduleOverlapPeriod).validate_python(response)


class ScheduleEndpoint(Endpoint):
    """Endpoint for work_schedule/schedules operations."""

    endpoint = 'work_schedule/schedules'

    async def all(self, **kwargs) -> ListApiResponse[WorkScheduleSchedule]:
        """Get all schedule records."""
        data = await self.api.get_all(self.endpoint, **kwargs)
        return ListApiResponse(raw_data=data, model_type=WorkScheduleSchedule)

    async def get(self, **kwargs) -> MetaApiResponse[WorkScheduleSchedule]:
        """Get schedules with pagination metadata."""
        query_params = kwargs.pop('params', {})
        query_params.setdefault('page', 1)
        response = await self.api.get(self.endpoint, params=query_params, **kwargs)
        return MetaApiResponse(raw_meta=response['meta'], raw_data=response['data'], model_type=WorkScheduleSchedule)

    async def get_by_id(self, schedule_id: int | str, **kwargs) -> WorkScheduleSchedule:
        """Get a specific schedule by ID."""
        data = await self.api.get(self.endpoint, schedule_id, **kwargs)
        return pydantic.TypeAdapter(WorkScheduleSchedule).validate_python(data)

    async def create(self, data: Mapping[str, typing.Any], **kwargs) -> WorkScheduleSchedule:
        """Create a new schedule."""
        response = await self.api.post(self.endpoint, json=data, **kwargs)
        return pydantic.TypeAdapter(WorkScheduleSchedule).validate_python(response)

    async def update(
        self,
        schedule_id: int | str,
        data: Mapping[str, typing.Any],
        **kwargs,
    ) -> WorkScheduleSchedule:
        """Update a schedule."""
        response = await self.api.put(self.endpoint, schedule_id, json=data, **kwargs)
        return pydantic.TypeAdapter(WorkScheduleSchedule).validate_python(response)

    async def toggle_archive(self, data: Mapping[str, typing.Any], **kwargs) -> WorkScheduleSchedule:
        """Toggle archive status of a schedule."""
        response = await self.api.post(self.endpoint, 'toggle_archive', json=data, **kwargs)
        return pydantic.TypeAdapter(WorkScheduleSchedule).validate_python(response)



import datetime
import typing
from collections.abc import Mapping, Sequence
from enum import StrEnum

import pydantic

from factorialhr._client import Endpoint, ListApiResponse, MetaApiResponse


class LocationType(StrEnum):
    office = 'office'
    business_trip = 'business_trip'
    work_from_home = 'work_from_home'


class TimeUnit(StrEnum):
    minute = 'minute'
    half_day = 'half_day'
    none = 'none'


class RequestType(StrEnum):
    """Enum for edit timesheet request types."""

    CREATE_SHIFT = 'create_shift'
    DELETE_SHIFT = 'delete_shift'
    UPDATE_SHIFT = 'update_shift'


class OpenShiftStatus(StrEnum):
    """Enum for open shift status."""

    OPENED = 'opened'
    CLOSED = 'closed'
    CREATED = 'created'


class OvertimeStatus(StrEnum):
    """Enum for overtime request status."""

    PENDING = 'pending'
    APPROVED = 'approved'
    REJECTED = 'rejected'
    NONE = 'none'


class HalfDay(StrEnum):
    """Enum for half day types."""

    BEGINNING_OF_DAY = 'beginning_of_day'
    END_OF_DAY = 'end_of_day'


class DayType(StrEnum):
    """Enum for day types."""

    SATURDAY = 'saturday'
    SUNDAY = 'sunday'
    BANK_HOLIDAY = 'bank_holiday'
    WORKDAY = 'workday'


class BreakConfiguration(pydantic.BaseModel):
    """Model for attendance_break_configuration."""

    model_config = pydantic.ConfigDict(frozen=True)

    id: int = pydantic.Field(description='Break configuration identifier')
    attendance_employees_setting_id: int = pydantic.Field(description='Id of the attendance employee setting')
    time_settings_break_configuration_id: int = pydantic.Field(
        description='Id of the time settings break configuration',
    )
    enabled: bool = pydantic.Field(description='Status of the break configuration if enabled or not')
    name: str | None = pydantic.Field(default=None, description='Name of the break configuration')
    paid: bool | None = pydantic.Field(default=None, description='Check the break configuration is paid or not')


class BreakConfigurationsEndpoint(Endpoint):
    """Endpoint for break configurations operations."""

    endpoint = 'attendance/break_configurations'

    async def all(self, **kwargs) -> ListApiResponse[BreakConfiguration]:
        """Get all break configurations records."""
        data = await self.api.get_all(self.endpoint, **kwargs)
        return ListApiResponse(model_type=BreakConfiguration, raw_data=data)

    async def get(self, **kwargs) -> MetaApiResponse[BreakConfiguration]:
        """Get break configurations with pagination metadata."""
        query_params = kwargs.pop('params', {})
        query_params.setdefault('page', 1)
        response = await self.api.get(self.endpoint, params=query_params, **kwargs)
        return MetaApiResponse(model_type=BreakConfiguration, raw_meta=response['meta'], raw_data=response['data'])

    async def get_by_id(self, break_configuration_id: int | str, **kwargs) -> BreakConfiguration:
        """Get a specific break configuration by ID."""
        data = await self.api.get(self.endpoint, break_configuration_id, **kwargs)
        return pydantic.TypeAdapter(BreakConfiguration).validate_python(data)

    async def create(self, data: Mapping[str, typing.Any], **kwargs) -> BreakConfiguration:
        """Create a new break configuration."""
        response = await self.api.post(self.endpoint, json=data, **kwargs)
        return pydantic.TypeAdapter(BreakConfiguration).validate_python(response)

    async def update(
        self,
        break_configuration_id: int | str,
        data: Mapping[str, typing.Any],
        **kwargs,
    ) -> BreakConfiguration:
        """Update a break configuration."""
        response = await self.api.put(self.endpoint, break_configuration_id, json=data, **kwargs)
        return pydantic.TypeAdapter(BreakConfiguration).validate_python(response)


class EditTimesheetRequest(pydantic.BaseModel):
    """Model for attendance_edit_timesheet_request."""

    model_config = pydantic.ConfigDict(frozen=True)

    id: int = pydantic.Field(description='Unique identifier for the edit timesheet request')
    approved: bool | None = pydantic.Field(default=None, description='Status of the edit timesheet request')
    request_type: RequestType = pydantic.Field(description='Type of the request')
    employee_id: int = pydantic.Field(description="Id of the shift's employee")
    workable: bool | None = pydantic.Field(default=None, description='Indicates if the shift is workable or a break')
    clock_in: datetime.time | None = pydantic.Field(default=None, description='Clock in of the shift')
    clock_out: datetime.time | None = pydantic.Field(default=None, description='Clock out of the shift')
    location_type: LocationType | None = pydantic.Field(default=None, description='Location of the shift')
    reason: str | None = pydantic.Field(default=None, description='Approve or reject reason')
    attendance_shift_id: int | None = pydantic.Field(default=None, description='Id of the shift for the request')
    time_settings_break_configuration_id: int | None = pydantic.Field(
        default=None,
        description='Id of the type of break for the request',
    )
    observations: str | None = pydantic.Field(default=None, description='Additional observations for the shift')
    date: datetime.date | None = pydantic.Field(default=None, description='Date of the shift')
    reference_date: datetime.date | None = pydantic.Field(default=None, description='Reference date for the shift')


class EditTimesheetRequestsEndpoint(Endpoint):
    """Endpoint for edit timesheet requests operations."""

    endpoint = 'attendance/edit_timesheet_requests'

    async def all(self, **kwargs) -> ListApiResponse[EditTimesheetRequest]:
        """Get all edit timesheet requests records."""
        data = await self.api.get_all(self.endpoint, **kwargs)
        return ListApiResponse(model_type=EditTimesheetRequest, raw_data=data)

    async def get(self, **kwargs) -> MetaApiResponse[EditTimesheetRequest]:
        """Get edit timesheet requests with pagination metadata."""
        query_params = kwargs.pop('params', {})
        query_params.setdefault('page', 1)
        response = await self.api.get(self.endpoint, params=query_params, **kwargs)
        return MetaApiResponse(model_type=EditTimesheetRequest, raw_meta=response['meta'], raw_data=response['data'])

    async def get_by_id(self, edit_timesheet_request_id: int | str, **kwargs) -> EditTimesheetRequest:
        """Get a specific edit timesheet request by ID."""
        data = await self.api.get(self.endpoint, edit_timesheet_request_id, **kwargs)
        return pydantic.TypeAdapter(EditTimesheetRequest).validate_python(data)

    async def create(self, data: Mapping[str, typing.Any], **kwargs) -> EditTimesheetRequest:
        """Create a new edit timesheet request."""
        response = await self.api.post(self.endpoint, json=data, **kwargs)
        return pydantic.TypeAdapter(EditTimesheetRequest).validate_python(response)

    async def update(
        self,
        edit_timesheet_request_id: int | str,
        data: Mapping[str, typing.Any],
        **kwargs,
    ) -> EditTimesheetRequest:
        """Update an edit timesheet request."""
        response = await self.api.put(self.endpoint, edit_timesheet_request_id, json=data, **kwargs)
        return pydantic.TypeAdapter(EditTimesheetRequest).validate_python(response)

    async def delete(self, edit_timesheet_request_id: int | str, **kwargs) -> EditTimesheetRequest:
        """Delete an edit timesheet request."""
        response = await self.api.delete(self.endpoint, edit_timesheet_request_id, **kwargs)
        return pydantic.TypeAdapter(EditTimesheetRequest).validate_python(response)


class EstimatedTime(pydantic.BaseModel):
    """Model for attendance_estimated_time."""

    model_config = pydantic.ConfigDict(frozen=True)

    date: datetime.date = pydantic.Field(description='Date of the estimated time')
    company_id: int = pydantic.Field(description='Company identifier')
    employee_id: int = pydantic.Field(description='Employee identifier')
    expected_minutes: float = pydantic.Field(
        description=(
            'Amount of minutes the employee has to work without taking into '
            'consideration time off leaves and bank holidays'
        ),
    )
    regular_minutes: float = pydantic.Field(description='Amount of regular minutes the employee has to work')
    overtime_minutes: float = pydantic.Field(
        description='Amount of overtime minutes the employee has to work (only available with Shift Management)',
    )
    breaks: Sequence[typing.Any] = pydantic.Field(description='List of breaks')
    time_unit: TimeUnit = pydantic.Field(description='Time unit for the estimation')
    estimated_half_days: int = pydantic.Field(description='Number of estimated half days')
    shifts: Sequence[typing.Any] = pydantic.Field(description='List of shifts')
    source: str = pydantic.Field(
        description="Source of the estimated time. Could be employee's contract, work schedule or shift management",
    )
    id: str = pydantic.Field(description='ID to specify the estimation time it includes the employee_id and date')
    minutes: float = pydantic.Field(description='Amount of minutes the employee has to work')


class OpenShift(pydantic.BaseModel):
    """Model for attendance_open_shift."""

    model_config = pydantic.ConfigDict(frozen=True)

    id: int = pydantic.Field(description='Open Shift identifier')
    employee_id: int = pydantic.Field(description='Employee identifier from the open shift')
    date: datetime.date = pydantic.Field(description='Date of the open shift')
    reference_date: datetime.date = pydantic.Field(description='Reference date for the shift')
    clock_in: datetime.datetime = pydantic.Field(description='Clock in time from the shift. Ignore the date part')
    clock_out: datetime.datetime | None = pydantic.Field(
        default=None,
        description='For open shifts, this field is null',
    )
    status: OpenShiftStatus = pydantic.Field(description='Status of the shift')
    workable: bool = pydantic.Field(description='Indicates if the shift is a break or a workable shift')
    automatic_clock_in: bool = pydantic.Field(description='Indicates if the shift is automatic or not')
    location_type: LocationType | None = pydantic.Field(
        default=None,
        description='String representing the location type of the shift. Examples work_from_home, office, etc',
    )
    workplace_id: int | None = pydantic.Field(
        default=None,
        description='Identifier for the workplace assigned to the shift',
    )
    time_settings_break_configuration_id: int | None = pydantic.Field(
        default=None,
        description='If the shift is a break, this field will have the break configuration id',
    )


class OvertimeRequest(pydantic.BaseModel):
    """Model for attendance_overtime_request."""

    model_config = pydantic.ConfigDict(frozen=True)

    id: int = pydantic.Field(description='Overtime request identifier')
    employee_id: int = pydantic.Field(description='Employee identifier')
    approver_id: int | None = pydantic.Field(default=None, description='Approver identifier')
    author_id: int = pydantic.Field(description='Author identifier')
    status: OvertimeStatus = pydantic.Field(description='Status of the overtime request')
    description: str | None = pydantic.Field(default=None, description='Description of the overtime request')
    reason: str | None = pydantic.Field(default=None, description='Reason for the overtime request')
    date: datetime.date = pydantic.Field(description='Date of the overtime request')
    hours_amount_in_cents: int = pydantic.Field(description='Hours amount in cents')
    created_at: datetime.datetime | None = pydantic.Field(
        default=None,
        description='Creation date of the overtime request',
    )
    approver: bool = pydantic.Field(description='Whether the request has an approver')
    approver_full_name: str | None = pydantic.Field(default=None, description='Full name of the approver')
    is_editable: bool = pydantic.Field(description='Defines if the overtime request can be edited')


class AttendanceShift(pydantic.BaseModel):
    """Model for attendance_shift."""

    model_config = pydantic.ConfigDict(frozen=True)

    id: int = pydantic.Field(description='Unique identifier for the shift')
    employee_id: int = pydantic.Field(description='Identifier for the employee assigned to the shift')
    date: datetime.date = pydantic.Field(description='Date of the shift')
    reference_date: datetime.date = pydantic.Field(description='Reference date for the shift')
    clock_in: datetime.time | None = pydantic.Field(default=None, description='Time when the employee clocked in')
    clock_out: datetime.time | None = pydantic.Field(default=None, description='Time when the employee clocked out')
    in_source: str | None = pydantic.Field(default=None, description='Source of the clock-in time')
    out_source: str | None = pydantic.Field(default=None, description='Source of the clock-out time')
    observations: str | None = pydantic.Field(default=None, description='Additional observations about the shift')
    location_type: LocationType | None = pydantic.Field(default=None, description='Type of location for the shift')
    half_day: HalfDay | None = pydantic.Field(default=None, description='Indicates which worked part of the day')
    in_location_latitude: float | None = pydantic.Field(default=None, description='Latitude of the clock-in location')
    in_location_longitude: float | None = pydantic.Field(default=None, description='Longitude of the clock-in location')
    in_location_accuracy: float | None = pydantic.Field(default=None, description='Accuracy of the clock-in location')
    out_location_latitude: float | None = pydantic.Field(default=None, description='Latitude of the clock-out location')
    out_location_longitude: float | None = pydantic.Field(
        default=None,
        description='Longitude of the clock-out location',
    )
    out_location_accuracy: float | None = pydantic.Field(default=None, description='Accuracy of the clock-out location')
    workable: bool | None = pydantic.Field(default=None, description='Indicates if the shift is workable')
    created_at: datetime.datetime = pydantic.Field(description='Timestamp when the shift record was created')
    workplace_id: int | None = pydantic.Field(default=None, description='Identifier for the location')
    time_settings_break_configuration_id: int | None = pydantic.Field(
        default=None,
        description='Identifier for the break configuration',
    )
    company_id: int = pydantic.Field(description='Identifier for the company')
    updated_at: datetime.datetime = pydantic.Field(description='Timestamp when the shift record was updated')
    minutes: int = pydantic.Field(description='Number in minutes of the shift')
    clock_in_with_seconds: datetime.time | None = pydantic.Field(default=None, description='Clock in time with seconds')


class WorkedTime(pydantic.BaseModel):
    """Model for attendance_worked_time."""

    model_config = pydantic.ConfigDict(frozen=True)

    employee_id: int = pydantic.Field(description='Employee identifier')
    date: datetime.date = pydantic.Field(description='Date of the worked time')
    company_id: int = pydantic.Field(description='Company identifier')
    tracked_minutes: int = pydantic.Field(description='Number of tracked minutes')
    multiplied_minutes: int = pydantic.Field(description='Number of multiplied minutes')
    pending_minutes: int = pydantic.Field(description='Number of pending minutes')
    minutes: int = pydantic.Field(description='Total number of minutes')
    time_unit: TimeUnit = pydantic.Field(description='Time unit for the worked time')
    worked_time_blocks: Sequence[typing.Any] = pydantic.Field(description='List of worked time blocks')
    day_type: DayType = pydantic.Field(description='Type of day')
    id: str = pydantic.Field(description='ID to specify the worked time it includes the employee_id and date')


class EstimatedTimesEndpoint(Endpoint):
    """Endpoint for estimated times operations."""

    endpoint = 'attendance/estimated_times'

    async def get(self, **kwargs) -> MetaApiResponse[EstimatedTime]:
        """Get estimated times with pagination metadata."""
        query_params = kwargs.pop('params', {})
        query_params.setdefault('page', 1)
        response = await self.api.get(self.endpoint, params=query_params, **kwargs)
        return MetaApiResponse(model_type=EstimatedTime, raw_meta=response['meta'], raw_data=response['data'])

    async def all(self, **kwargs) -> ListApiResponse[EstimatedTime]:
        """Get all estimated times records."""
        data = await self.api.get_all(self.endpoint, **kwargs)
        return ListApiResponse(model_type=EstimatedTime, raw_data=data)


class OpenShiftsEndpoint(Endpoint):
    """Endpoint for open shifts operations."""

    endpoint = 'attendance/open_shifts'

    async def get(self, **kwargs) -> MetaApiResponse[OpenShift]:
        """Get open shifts with pagination metadata."""
        query_params = kwargs.pop('params', {})
        query_params.setdefault('page', 1)
        response = await self.api.get(self.endpoint, params=query_params, **kwargs)
        return MetaApiResponse(model_type=OpenShift, raw_meta=response['meta'], raw_data=response['data'])

    async def all(self, **kwargs) -> ListApiResponse[OpenShift]:
        """Get all open shifts records."""
        data = await self.api.get_all(self.endpoint, **kwargs)
        return ListApiResponse(model_type=OpenShift, raw_data=data)


class OvertimeRequestsEndpoint(Endpoint):
    """Endpoint for overtime requests operations."""

    endpoint = 'attendance/overtime_requests'

    async def all(self, **kwargs) -> ListApiResponse[OvertimeRequest]:
        """Get all overtime requests records."""
        data = await self.api.get_all(self.endpoint, **kwargs)
        return ListApiResponse(model_type=OvertimeRequest, raw_data=data)

    async def get(self, **kwargs) -> MetaApiResponse[OvertimeRequest]:
        """Get overtime requests with pagination metadata."""
        query_params = kwargs.pop('params', {})
        query_params.setdefault('page', 1)
        response = await self.api.get(self.endpoint, params=query_params, **kwargs)
        return MetaApiResponse(model_type=OvertimeRequest, raw_meta=response['meta'], raw_data=response['data'])

    async def get_by_id(self, overtime_request_id: int | str, **kwargs) -> OvertimeRequest:
        """Get a specific overtime request by ID."""
        data = await self.api.get(self.endpoint, overtime_request_id, **kwargs)
        return pydantic.TypeAdapter(OvertimeRequest).validate_python(data)

    async def create(self, data: Mapping[str, typing.Any], **kwargs) -> OvertimeRequest:
        """Create a new overtime request."""
        response = await self.api.post(self.endpoint, json=data, **kwargs)
        return pydantic.TypeAdapter(OvertimeRequest).validate_python(response)

    async def update(
        self,
        overtime_request_id: int | str,
        data: Mapping[str, typing.Any],
        **kwargs,
    ) -> OvertimeRequest:
        """Update an overtime request."""
        response = await self.api.put(self.endpoint, overtime_request_id, json=data, **kwargs)
        return pydantic.TypeAdapter(OvertimeRequest).validate_python(response)

    async def delete(self, overtime_request_id: int | str, **kwargs) -> OvertimeRequest:
        """Delete an overtime request."""
        response = await self.api.delete(self.endpoint, overtime_request_id, **kwargs)
        return pydantic.TypeAdapter(OvertimeRequest).validate_python(response)

    async def approve(self, data: Mapping[str, typing.Any], **kwargs) -> OvertimeRequest:
        """Approve an overtime request."""
        response = await self.api.post(self.endpoint, 'approve', json=data, **kwargs)
        return pydantic.TypeAdapter(OvertimeRequest).validate_python(response)

    async def reject(self, data: Mapping[str, typing.Any], **kwargs) -> OvertimeRequest:
        """Reject an overtime request."""
        response = await self.api.post(self.endpoint, 'reject', json=data, **kwargs)
        return pydantic.TypeAdapter(OvertimeRequest).validate_python(response)


class ShiftsEndpoint(Endpoint):
    """Endpoint for shifts operations."""

    endpoint = 'attendance/shifts'

    async def all(self, **kwargs) -> ListApiResponse[AttendanceShift]:
        """Get all shifts records."""
        data = await self.api.get_all(self.endpoint, **kwargs)
        return ListApiResponse(raw_data=data, model_type=AttendanceShift)

    async def get(self, **kwargs) -> MetaApiResponse[AttendanceShift]:
        """Get shifts with pagination metadata."""
        query_params = kwargs.pop('params', {})
        query_params.setdefault('page', 1)
        response = await self.api.get(self.endpoint, params=query_params, **kwargs)
        return MetaApiResponse(raw_meta=response['meta'], raw_data=response['data'], model_type=AttendanceShift)

    async def get_by_id(self, shift_id: int | str, **kwargs) -> AttendanceShift:
        """Get a specific shift by ID."""
        data = await self.api.get(self.endpoint, shift_id, **kwargs)
        return pydantic.TypeAdapter(AttendanceShift).validate_python(data)

    async def create(self, data: Mapping[str, typing.Any], **kwargs) -> AttendanceShift:
        """Create a new shift."""
        response = await self.api.post(self.endpoint, json=data, **kwargs)
        return pydantic.TypeAdapter(AttendanceShift).validate_python(response)

    async def update(
        self,
        shift_id: int | str,
        data: Mapping[str, typing.Any],
        **kwargs,
    ) -> AttendanceShift:
        """Update a shift."""
        response = await self.api.put(self.endpoint, shift_id, json=data, **kwargs)
        return pydantic.TypeAdapter(AttendanceShift).validate_python(response)

    async def delete(self, shift_id: int | str, **kwargs) -> AttendanceShift:
        """Delete a shift."""
        response = await self.api.delete(self.endpoint, shift_id, **kwargs)
        return pydantic.TypeAdapter(AttendanceShift).validate_python(response)

    async def autofill(self, data: Mapping[str, typing.Any], **kwargs) -> Sequence[AttendanceShift]:
        """Autofill shifts."""
        response = await self.api.post(self.endpoint, 'autofill', json=data, **kwargs)
        return pydantic.TypeAdapter(list[AttendanceShift]).validate_python(response)

    async def break_start(self, data: Mapping[str, typing.Any], **kwargs) -> AttendanceShift:
        """Start a break in a shift."""
        response = await self.api.post(self.endpoint, 'break_start', json=data, **kwargs)
        return pydantic.TypeAdapter(AttendanceShift).validate_python(response)

    async def break_end(self, data: Mapping[str, typing.Any], **kwargs) -> AttendanceShift:
        """End a break in a shift."""
        response = await self.api.post(self.endpoint, 'break_end', json=data, **kwargs)
        return pydantic.TypeAdapter(AttendanceShift).validate_python(response)

    async def clock_in(self, data: Mapping[str, typing.Any], **kwargs) -> AttendanceShift:
        """Clock in a shift."""
        response = await self.api.post(self.endpoint, 'clock_in', json=data, **kwargs)
        return pydantic.TypeAdapter(AttendanceShift).validate_python(response)

    async def clock_out(self, data: Mapping[str, typing.Any], **kwargs) -> AttendanceShift:
        """Clock out a shift."""
        response = await self.api.post(self.endpoint, 'clock_out', json=data, **kwargs)
        return pydantic.TypeAdapter(AttendanceShift).validate_python(response)

    async def toggle_clock(self, data: Mapping[str, typing.Any], **kwargs) -> AttendanceShift:
        """Toggle clock (clock in/out) a shift."""
        response = await self.api.post(self.endpoint, 'toggle_clock', json=data, **kwargs)
        return pydantic.TypeAdapter(AttendanceShift).validate_python(response)


class Review(pydantic.BaseModel):
    """Model for attendance_review."""

    model_config = pydantic.ConfigDict(frozen=True)

    id: int = pydantic.Field(description='Review identifier')
    employee_id: int = pydantic.Field(description='Employee identifier')
    date: datetime.date = pydantic.Field(description='Review date')
    approved: bool = pydantic.Field(description='Whether the review is approved')
    created_at: datetime.datetime = pydantic.Field(description='Creation date')
    updated_at: datetime.datetime = pydantic.Field(description='Last update date')


class ReviewsEndpoint(Endpoint):
    """Endpoint for attendance reviews operations."""

    endpoint = 'attendance/reviews'

    async def all(self, **kwargs) -> ListApiResponse[Review]:
        """Get all reviews records."""
        data = await self.api.get_all(self.endpoint, **kwargs)
        return ListApiResponse(model_type=Review, raw_data=data)

    async def get(self, **kwargs) -> MetaApiResponse[Review]:
        """Get reviews with pagination metadata."""
        query_params = kwargs.pop('params', {})
        query_params.setdefault('page', 1)
        response = await self.api.get(self.endpoint, params=query_params, **kwargs)
        return MetaApiResponse(model_type=Review, raw_meta=response['meta'], raw_data=response['data'])

    async def get_by_id(self, review_id: int | str, **kwargs) -> Review:
        """Get a specific review by ID."""
        data = await self.api.get(self.endpoint, review_id, **kwargs)
        return pydantic.TypeAdapter(Review).validate_python(data)

    async def bulk_create(self, data: Mapping[str, typing.Any], **kwargs) -> Sequence[Review]:
        """Bulk create reviews."""
        response = await self.api.post(self.endpoint, 'bulk_create', json=data, **kwargs)
        return pydantic.TypeAdapter(list[Review]).validate_python(response)

    async def bulk_destroy(self, data: Mapping[str, typing.Any], **kwargs) -> Sequence[Review]:
        """Bulk destroy reviews."""
        response = await self.api.post(self.endpoint, 'bulk_destroy', json=data, **kwargs)
        return pydantic.TypeAdapter(list[Review]).validate_python(response)


class WorkedTimesEndpoint(Endpoint):
    """Endpoint for worked times operations."""

    endpoint = 'attendance/worked_times'

    async def get(self, **kwargs) -> MetaApiResponse[WorkedTime]:
        """Get worked times with pagination metadata."""
        query_params = kwargs.pop('params', {})
        query_params.setdefault('page', 1)
        response = await self.api.get(self.endpoint, params=query_params, **kwargs)
        return MetaApiResponse(model_type=WorkedTime, raw_meta=response['meta'], raw_data=response['data'])

    async def all(self, **kwargs) -> ListApiResponse[WorkedTime]:
        """Get all worked times records."""
        data = await self.api.get_all(self.endpoint, **kwargs)
        return ListApiResponse(model_type=WorkedTime, raw_data=data)



import datetime
import typing
from collections.abc import Mapping, Sequence

import pydantic

from factorialhr._client import Endpoint, ListApiResponse, MetaApiResponse


class Schema(pydantic.BaseModel):
    """Model for custom_resources_schema."""

    model_config = pydantic.ConfigDict(frozen=True)

    id: int = pydantic.Field(description='Schema identifier')
    name: str = pydantic.Field(description='Schema name')
    company_id: int = pydantic.Field(description='Company identifier where this schema belongs')
    hidden: bool = pydantic.Field(description='Manages visibility of the schema')
    position: int | None = pydantic.Field(default=None, description='Schema position within employee profile')


class Resource(pydantic.BaseModel):
    """Model for custom_resources_resource."""

    model_config = pydantic.ConfigDict(frozen=True)

    id: int = pydantic.Field(description='Resource identifier')
    name: str = pydantic.Field(description='Resource name')
    company_id: int = pydantic.Field(description='Company identifier')
    created_at: datetime.datetime = pydantic.Field(description='Creation date')
    updated_at: datetime.datetime = pydantic.Field(description='Last update date')


class CustomResourcesValue(pydantic.BaseModel):
    """Model for custom_resources_value."""

    model_config = pydantic.ConfigDict(frozen=True)

    id: int = pydantic.Field(description='Value identifier')
    field_id: int = pydantic.Field(description='Identifier of the field this value belongs to')
    attachable_id: int | None = pydantic.Field(
        default=None,
        description='The id of the attached resource like an employee',
    )
    long_text_value: str | None = pydantic.Field(
        default=None,
        description="When the field's type is long_text_value, value for schema long_text_value custom field",
    )
    date_value: datetime.date | None = pydantic.Field(
        default=None,
        description="When the field's type is date_value, value for schema date_value custom field",
    )
    text_value: str | None = pydantic.Field(
        default=None,
        description="When the field's type is text_value, value for schema text_value custom field",
    )
    number_value: int | None = pydantic.Field(
        default=None,
        description="When the field's type is number_value, value for schema number_value custom field",
    )
    option_value: str | None = pydantic.Field(
        default=None,
        description="When the field's type is option_value, selected value for schema option_value custom field",
    )
    cents_value: int | None = pydantic.Field(
        default=None,
        description="When the field's type is cents_value, value for schema cents_value custom field",
    )
    boolean_value: bool | None = pydantic.Field(
        default=None,
        description="When the field's type is boolean_value, value for schema boolean_value custom field",
    )
    single_choice_value: str | None = pydantic.Field(
        default=None,
        description=(
            "When the field's type is single_choice_value, selected value for schema single_choice_value custom field"
        ),
    )
    multiple_choice_value: Sequence[str] | None = pydantic.Field(
        default=None,
        description=(
            "When the field's type is multiple_choice_value, "
            'selected values for schema multiple_choice_value custom field'
        ),
    )


class CustomResourcesSchemasEndpoint(Endpoint):
    """Endpoint for custom_resources/schemas operations."""

    endpoint = 'custom_resources/schemas'

    async def all(self, **kwargs) -> ListApiResponse[Schema]:
        """Get all schemas records."""
        data = await self.api.get_all(self.endpoint, **kwargs)
        return ListApiResponse(raw_data=data, model_type=Schema)

    async def get(self, **kwargs) -> MetaApiResponse[Schema]:
        """Get schemas with pagination metadata."""
        query_params = kwargs.pop('params', {})
        query_params.setdefault('page', 1)
        response = await self.api.get(self.endpoint, params=query_params, **kwargs)
        return MetaApiResponse(raw_meta=response['meta'], raw_data=response['data'], model_type=Schema)

    async def get_by_id(self, schema_id: int | str, **kwargs) -> Schema:
        """Get a specific schema by ID."""
        data = await self.api.get(self.endpoint, schema_id, **kwargs)
        return pydantic.TypeAdapter(Schema).validate_python(data)

    async def create(self, data: Mapping[str, typing.Any], **kwargs) -> Schema:
        """Create a new schema."""
        response = await self.api.post(self.endpoint, json=data, **kwargs)
        return pydantic.TypeAdapter(Schema).validate_python(response)


class ResourcesEndpoint(Endpoint):
    """Endpoint for custom_resources/resources operations."""

    endpoint = 'custom_resources/resources'

    async def all(self, **kwargs) -> ListApiResponse[Resource]:
        """Get all resources records."""
        data = await self.api.get_all(self.endpoint, **kwargs)
        return ListApiResponse(raw_data=data, model_type=Resource)

    async def get(self, **kwargs) -> MetaApiResponse[Resource]:
        """Get resources with pagination metadata."""
        query_params = kwargs.pop('params', {})
        query_params.setdefault('page', 1)
        response = await self.api.get(self.endpoint, params=query_params, **kwargs)
        return MetaApiResponse(raw_meta=response['meta'], raw_data=response['data'], model_type=Resource)

    async def get_by_id(self, resource_id: int | str, **kwargs) -> Resource:
        """Get a specific resource by ID."""
        data = await self.api.get(self.endpoint, resource_id, **kwargs)
        return pydantic.TypeAdapter(Resource).validate_python(data)


class CustomResourcesValuesEndpoint(Endpoint):
    """Endpoint for custom_resources/values operations."""

    endpoint = 'custom_resources/values'

    async def all(self, **kwargs) -> ListApiResponse[CustomResourcesValue]:
        """Get all values records."""
        data = await self.api.get_all(self.endpoint, **kwargs)
        return ListApiResponse(raw_data=data, model_type=CustomResourcesValue)

    async def get(self, **kwargs) -> MetaApiResponse[CustomResourcesValue]:
        """Get values with pagination metadata."""
        query_params = kwargs.pop('params', {})
        query_params.setdefault('page', 1)
        response = await self.api.get(self.endpoint, params=query_params, **kwargs)
        return MetaApiResponse(raw_meta=response['meta'], raw_data=response['data'], model_type=CustomResourcesValue)

    async def get_by_id(self, value_id: int | str, **kwargs) -> CustomResourcesValue:
        """Get a specific value by ID."""
        data = await self.api.get(self.endpoint, value_id, **kwargs)
        return pydantic.TypeAdapter(CustomResourcesValue).validate_python(data)

    async def create(self, data: Mapping[str, typing.Any], **kwargs) -> CustomResourcesValue:
        """Create a new value."""
        response = await self.api.post(self.endpoint, json=data, **kwargs)
        return pydantic.TypeAdapter(CustomResourcesValue).validate_python(response)



import datetime
import typing
from collections.abc import Mapping, Sequence
from enum import StrEnum

import pydantic

from factorialhr._client import Endpoint, ListApiResponse, MetaApiResponse


class TaskStatus(StrEnum):
    """Enum for task status."""

    TODO = 'todo'
    IN_PROGRESS = 'in_progress'
    DONE = 'done'
    DISCARDED = 'discarded'


class Task(pydantic.BaseModel):
    """Model for tasks_task."""

    model_config = pydantic.ConfigDict(frozen=True)

    id: int = pydantic.Field(description='Identifier of the task')
    name: str = pydantic.Field(description='Name of the task')
    company_id: int = pydantic.Field(description='Company identifier of the author of the task')
    content: str | None = pydantic.Field(default=None, description='Content of the task')
    due_on: datetime.date | None = pydantic.Field(default=None, description='Due on date of the task')
    assignee_ids: Sequence[int] = pydantic.Field(
        description='Employees assigned to the task, assignee_id references to access_id',
    )
    author_employee_id: int | None = pydantic.Field(default=None, description='Employee id of the author of the task')
    completed_at: str | None = pydantic.Field(default=None, description='Completed at date of the task')
    completed_by_id: int | None = pydantic.Field(default=None, description='Completed by identifier')
    created_at: datetime.datetime = pydantic.Field(description='Created at date of the task')
    updated_at: datetime.datetime = pydantic.Field(description='Updated at date of the task')
    status: TaskStatus | None = pydantic.Field(default=None, description='Status of the task')


class TaskFile(pydantic.BaseModel):
    """Model for tasks_task_file."""

    model_config = pydantic.ConfigDict(frozen=True)

    id: int = pydantic.Field(description='Identifier of the file')
    task_id: int = pydantic.Field(description='Identifier of the task')
    filename: str = pydantic.Field(description='Name of the file')
    content_type: str | None = pydantic.Field(default=None, description='Content type of the file')
    path: str = pydantic.Field(description='Path of the file, for downloading the file you need to concat api_url/path')
    created_at: datetime.datetime = pydantic.Field(description='Creation date of the file')


class TasksEndpoint(Endpoint):
    """Endpoint for tasks/tasks operations."""

    endpoint = 'tasks/tasks'

    async def all(self, **kwargs) -> ListApiResponse[Task]:
        """Get all tasks records."""
        data = await self.api.get_all(self.endpoint, **kwargs)
        return ListApiResponse(model_type=Task, raw_data=data)

    async def get(self, **kwargs) -> MetaApiResponse[Task]:
        """Get tasks with pagination metadata."""
        query_params = kwargs.pop('params', {})
        query_params.setdefault('page', 1)
        response = await self.api.get(self.endpoint, params=query_params, **kwargs)
        return MetaApiResponse(model_type=Task, raw_meta=response['meta'], raw_data=response['data'])

    async def get_by_id(self, task_id: int | str, **kwargs) -> Task:
        """Get a specific task by ID."""
        data = await self.api.get(self.endpoint, task_id, **kwargs)
        return pydantic.TypeAdapter(Task).validate_python(data)

    async def create(self, data: Mapping[str, typing.Any], **kwargs) -> Task:
        """Create a new task."""
        response = await self.api.post(self.endpoint, json=data, **kwargs)
        return pydantic.TypeAdapter(Task).validate_python(response)

    async def update(self, task_id: int | str, data: Mapping[str, typing.Any], **kwargs) -> Task:
        """Update a task."""
        response = await self.api.put(self.endpoint, task_id, json=data, **kwargs)
        return pydantic.TypeAdapter(Task).validate_python(response)

    async def delete(self, task_id: int | str, **kwargs) -> Task:
        """Delete a task."""
        response = await self.api.delete(self.endpoint, task_id, **kwargs)
        return pydantic.TypeAdapter(Task).validate_python(response)

    async def bulk_create(self, data: Mapping[str, typing.Any], **kwargs) -> Sequence[Task]:
        """Bulk create tasks."""
        response = await self.api.post(self.endpoint, 'bulk_create', json=data, **kwargs)
        return pydantic.TypeAdapter(list[Task]).validate_python(response)

    async def bulk_delete(self, data: Mapping[str, typing.Any], **kwargs) -> Sequence[Task]:
        """Bulk delete tasks."""
        response = await self.api.post(self.endpoint, 'bulk_delete', json=data, **kwargs)
        return pydantic.TypeAdapter(list[Task]).validate_python(response)

    async def bulk_update(self, data: Mapping[str, typing.Any], **kwargs) -> Sequence[Task]:
        """Bulk update tasks."""
        response = await self.api.post(self.endpoint, 'bulk_update', json=data, **kwargs)
        return pydantic.TypeAdapter(list[Task]).validate_python(response)

    async def copy(self, data: Mapping[str, typing.Any], **kwargs) -> Task:
        """Copy a task."""
        response = await self.api.post(self.endpoint, 'copy', json=data, **kwargs)
        return pydantic.TypeAdapter(Task).validate_python(response)

    async def create_comment(self, data: Mapping[str, typing.Any], **kwargs) -> Task:
        """Create a comment on a task."""
        response = await self.api.post(self.endpoint, 'create_comment', json=data, **kwargs)
        return pydantic.TypeAdapter(Task).validate_python(response)


class TaskFilesEndpoint(Endpoint):
    """Endpoint for tasks/task_files operations."""

    endpoint = 'tasks/task_files'

    async def all(self, **kwargs) -> ListApiResponse[TaskFile]:
        """Get all task files records."""
        data = await self.api.get_all(self.endpoint, **kwargs)
        return ListApiResponse(model_type=TaskFile, raw_data=data)

    async def get(self, **kwargs) -> MetaApiResponse[TaskFile]:
        """Get task files with pagination metadata."""
        query_params = kwargs.pop('params', {})
        query_params.setdefault('page', 1)
        response = await self.api.get(self.endpoint, params=query_params, **kwargs)
        return MetaApiResponse(model_type=TaskFile, raw_meta=response['meta'], raw_data=response['data'])

    async def get_by_id(self, file_id: int | str, **kwargs) -> TaskFile:
        """Get a specific task file by ID."""
        data = await self.api.get(self.endpoint, file_id, **kwargs)
        return pydantic.TypeAdapter(TaskFile).validate_python(data)

    async def create(self, data: Mapping[str, typing.Any], **kwargs) -> TaskFile:
        """Create a new task file."""
        response = await self.api.post(self.endpoint, json=data, **kwargs)
        return pydantic.TypeAdapter(TaskFile).validate_python(response)

    async def delete(self, file_id: int | str, **kwargs) -> TaskFile:
        """Delete a task file."""
        response = await self.api.delete(self.endpoint, file_id, **kwargs)
        return pydantic.TypeAdapter(TaskFile).validate_python(response)
